from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from DIRAC.Core.Workflow.Parameter import *
from DIRAC.Core.Workflow.Module import *
from DIRAC.Core.Workflow.Step import *
from DIRAC.Core.Workflow.Workflow import *
from DIRAC.Core.Workflow.WorkflowReader import *

# define Module 2
module2 = ModuleDefinition("GaudiApplication")  # during constraction class creates duplicating copies of the params
module2.setDescription("Gaudi Application module")
module2.setBody("from WorkflowLib.Module.GaudiApplication import GaudiApplication\n")

# we add empty parameters but linked up as default
module2.addParameter(Parameter("DataType", "", "string", "self", "DataType", True, False, "data type"))
module2.addParameter(Parameter("CONFIG_NAME", "", "string", "self", "CONFIG_NAME", True, False, "Configuration Name"))
module2.addParameter(
    Parameter("CONFIG_VERSION", "", "string", "self", "CONFIG_VERSION", True, False, "Configuration Version")
)
module2.addParameter(Parameter("EVENTTYPE", "", "string", "self", "EVENTTYPE", True, False, "Event Type"))
module2.addParameter(Parameter("appName", "", "string", "self", "appName", True, False, "Application Name"))
module2.addParameter(Parameter("appVersion", "", "string", "self", "appVersion", True, False, "Application Version"))
module2.addParameter(Parameter("appType", "", "string", "self", "appType", True, False, "Application Version"))
module2.addParameter(Parameter("appLog", "", "string", "self", "appLog", True, False, "list of logfile"))
module2.addParameter(Parameter("inputData", "", "jdl", "self", "inputData", True, False, "List of InputData"))
module2.addParameter(Parameter("inputDataType", "", "string", "self", "inputDataType", True, False, "Input Data Type"))
module2.addParameter(
    Parameter("NUMBER_OF_EVENTS", "", "string", "self", "NUMBER_OF_EVENTS", True, False, "number of events requested")
)
module2.addParameter(Parameter("optionsFile", "", "string", "self", "optionsFile", True, False, "Options File"))
module2.addParameter(
    Parameter("optionsLine", "", "string", "self", "optionsLine", True, False, "Number of Event", "option")
)
module2.addParameter(Parameter("systemConfig", "", "string", "self", "systemConfig", True, False, "Job Platform"))
module2.addParameter(Parameter("poolXMLCatName", "", "string", "self", "poolXMLCatName", True, False, "POOL XML slice"))

# define module 3
module3 = ModuleDefinition("LogChecker")  # during constraction class creates duplicating copies of the params
module3.setDescription("Check LogFile module")
module3.setBody("from WorkflowLib.Module.LogChecker import *\n")

# we add parameters and link them to the level of step
module3.addParameter(Parameter("DataType", "", "string", "self", "DataType", True, False, "data type"))
module3.addParameter(Parameter("SourceData", "", "string", "self", "SourceData", True, False, "InputData"))
module3.addParameter(Parameter("CONFIG_NAME", "", "string", "self", "CONFIG_NAME", True, False, "Configuration Name"))
module3.addParameter(
    Parameter("CONFIG_VERSION", "", "string", "self", "CONFIG_VERSION", True, False, "Configuration Version")
)
module3.addParameter(
    Parameter("NUMBER_OF_EVENTS", "", "string", "self", "NUMBER_OF_EVENTS", True, False, "number of events requested")
)
module3.addParameter(Parameter("NUMBER_OF_EVENTS_INPUT", "", "string", "", "", True, True, "number of events as input"))
module3.addParameter(
    Parameter("NUMBER_OF_EVENTS_OUTPUT", "", "string", "", "", False, True, "number of events as input")
)
module3.addParameter(Parameter("appName", "", "string", "self", "appName", True, False, "Application Name"))
module3.addParameter(Parameter("appVersion", "", "string", "self", "appVersion", True, False, "Application Version"))
module3.addParameter(Parameter("appType", "", "string", "self", "appType", True, False, "Application Version"))
module3.addParameter(Parameter("appLog", "", "string", "self", "appLog", True, False, "list of logfile"))
module3.addParameter(Parameter("poolXMLCatName", "", "string", "self", "poolXMLCatName", True, False, "POOL XML slice"))
module3.addParameter(Parameter("inputData", "", "string", "self", "inputData", True, False, "InputData"))
module3.addParameter(
    Parameter("OUTPUT_MAX", "", "string", "self", "OUTPUT_MAX", True, False, "nb max of output to keep")
)
module3.addParameter(Parameter("EMAIL", "@{EMAILNAME}", "string", "", "", True, False, "EMAIL adress"))

# define module 4
module4 = ModuleDefinition("BookkeepingReport")
module4.setDescription("Bookkeeping Report module")
module4.setBody("from WorkflowLib.Module.BookkeepingReport import * \n")
module4.addParameter(Parameter("STEP_ID", "", "string", "self", "STEP_ID", True, False, "EMAIL adress"))
module4.addParameter(
    Parameter("NUMBER_OF_EVENTS", "", "string", "self", "NUMBER_OF_EVENTS", True, False, "number of events requested")
)
module4.addParameter(
    Parameter("NUMBER_OF_EVENTS_INPUT", "", "string", "", "", True, False, "number of events as input")
)
module4.addParameter(
    Parameter("NUMBER_OF_EVENTS_OUTPUT", "", "string", "", "", True, False, "number of events as input")
)
module4.addParameter(Parameter("DataType", "", "string", "self", "DataType", True, False, "data type"))
module4.addParameter(Parameter("CONFIG_NAME", "", "string", "self", "CONFIG_NAME", True, False, "Configuration Name"))
module4.addParameter(
    Parameter("CONFIG_VERSION", "", "string", "self", "CONFIG_VERSION", True, False, "Configuration Version")
)
module4.addParameter(Parameter("appName", "", "string", "self", "appName", True, False, "Application Name"))
module4.addParameter(Parameter("appVersion", "", "string", "self", "appVersion", True, False, "Application Version"))
module4.addParameter(Parameter("inputData", "", "string", "self", "inputData", True, False, "InputData"))
module4.addParameter(Parameter("inputDataType", "", "string", "self", "inputDataType", True, False, "Input Data Type"))
module4.addParameter(Parameter("EVENTTYPE", "", "string", "self", "EVENTTYPE", True, False, "Event Type"))
module4.addParameter(Parameter("appType", "", "string", "self", "appType", True, False, "Application Version"))
module4.addParameter(Parameter("poolXMLCatName", "", "string", "self", "poolXMLCatName", True, False, "POOL XML slice"))
module4.addParameter(Parameter("SourceData", "", "string", "self", "SourceData", True, False, "InputData"))
module4.addParameter(Parameter("appLog", "", "string", "self", "appLog", True, False, "list of logfile"))
module4.addParameter(Parameter("listoutput", [], "list", "self", "listoutput", True, False, "list of output data"))


# define module 5
module5 = ModuleDefinition("StepFinalization")
module5.setDescription("Step Finalization module")
module5.setBody("from WorkflowLib.Module.StepFinalization import * \n")
module5.addParameter(Parameter("listoutput", [], "list", "self", "listoutput", True, False, "list of output data"))
module5.addParameter(Parameter("poolXMLCatName", "", "string", "self", "poolXMLCatName", True, False, "POOL XML slice"))
module5.addParameter(Parameter("inputData", "", "string", "self", "inputData", True, False, "InputData"))
module5.addParameter(Parameter("SourceData", "", "string", "self", "SourceData", True, False, "InputData"))

# define module 6
module6 = ModuleDefinition("JobFinalization")
module6.setDescription("Job Finalization module")
module6.setBody("from WorkflowLib.Module.JobFinalization import * \n")
# module6.addParameter(Parameter("listoutput",[],"list","self","listoutput",True,False,"list of output data"))
# module6.addParameter(Parameter("outputData",'',"string","self","outputData",True,False,"list of output data"))
module6.addParameter(Parameter("poolXMLCatName", "", "string", "self", "poolXMLCatName", True, False, "POOL XML slice"))
# KGG Some problem with InputData and with inputData - it is not clear for me
# module6.addParameter(Parameter("inputData","","string","self","InputData",True,False,"InputData"))
module6.addParameter(Parameter("SourceData", "", "string", "self", "SourceData", True, False, "InputData"))

# module6.addParameter(Parameter("listoutput",[],"list","self","listoutput",True,False,"list of output data"))
# I am adding those parameters with empty list
# used one going to be replaced on the level of the step
module6.addParameter(Parameter("listoutput_1", [], "list", "self", "listoutput_1", True, False, "list of output data"))
module6.addParameter(Parameter("listoutput_2", [], "list", "self", "listoutput_2", True, False, "list of output data"))
module6.addParameter(Parameter("listoutput_3", [], "list", "self", "listoutput_3", True, False, "list of output data"))
module6.addParameter(Parameter("listoutput_4", [], "list", "self", "listoutput_4", True, False, "list of output data"))
module6.addParameter(Parameter("listoutput_5", [], "list", "self", "listoutput_5", True, False, "list of output data"))
module6.addParameter(Parameter("listoutput_6", [], "list", "self", "listoutput_6", True, False, "list of output data"))

# define module 7
# module7 = ModuleDefinition('Dummy')
# module7.setDescription('Dummy module')
# module7.setBody('from WorkflowLib.Module.Dummy import * \n')


###############   STEPS ##################################
# step 1 we are creating step definition
step1 = StepDefinition("Gaudi_App_Step")
step1.addModule(module2)  # Creating instance of the module 'Gaudi_App_Step'
moduleInstance2 = step1.createModuleInstance("GaudiApplication", "module2")
# moduleInstance2 = step1.createModuleInstance('Dummy', 'module7')

step1.addModule(module3)  # Creating instance of the module 'LogChecker'
moduleInstance3 = step1.createModuleInstance("LogChecker", "module3")

step1.addModule(module4)  # Creating instance of the module 'LogChecker'
moduleInstance4 = step1.createModuleInstance("BookkeepingReport", "module4")
moduleInstance4.setLink("NUMBER_OF_EVENTS_INPUT", moduleInstance3.getName(), "NUMBER_OF_EVENTS_INPUT")
moduleInstance4.setLink("NUMBER_OF_EVENTS_OUTPUT", moduleInstance3.getName(), "NUMBER_OF_EVENTS_OUTPUT")
# in principle we can link parameters of moduleInstance2 with moduleInstance1 but
# in this case we going to use link with the step
# step1.addModule(module5)
# moduleInstance5 = step1.createModuleInstance('StepFinalization','module5')

# now we can add parameters for the STEP but instead of typing them we can just use old one from modules
step1.addParameterLinked(module3.parameters)
step1.addParameterLinked(module2.parameters)

# and we can add additional parameter which will be used as a global
step1.addParameter(
    Parameter("STEP_ID", "@{PRODUCTION_ID}_@{JOB_ID}_@{STEP_NUMBER}", "string", "", "", True, False, "Temporary fix")
)
step1.addParameter(Parameter("EVENTTYPE", "30000000", "string", "", "", True, False, "Event Type"))
step1.addParameter(Parameter("outputData", "@{STEP_ID}.root", "string", "", "", True, False, "etc name"))
step1.addParameter(Parameter("etcf", "SETC_@{STEP_ID}.root", "string", "", "", True, False, "etc name"))
step1.setValue("appLog", "@{appName}_@{PRODUCTION_ID}_@{JOB_ID}_@{STEP_NUMBER}.log")
step1.unlink(["appLog", "appName", "appType"])
step1.unlink(["poolXMLCatName", "SourceData", "DataType", "CONFIG_NAME", "CONFIG_VERSION"])
step1.addParameter(Parameter("listoutput", [], "list", "", "", True, False, "list of output data"))

# outputData = "@{PRODUCTION_ID}_@{JOB_ID}_@{STEP_NUMBER}.@{appType}"
opt_dav = "EvtTupleSvc.Output = {}"
opt_dav = opt_dav + ";ApplicationMgr.OutStream -= {'DstWriter'}"
opt_dav = opt_dav + ";ApplicationMgr.OutStream = {'Sequencer/SeqWriteTag'}"
opt_dav = opt_dav + ';ApplicationMgr.TopAlg -= { "GaudiSequencer/SeqPreselHWZ2bbl" }'
opt_dav = opt_dav + ";MessageSvc.Format = '%u % F%18W%S%7W%R%T %0W%M';MessageSvc.timeFormat = '%Y-%m-%d %H:%M:%S UTC'"
opt_dav = (
    opt_dav
    + ";WR.Output = \"Collection='EVTTAGS/TagCreator/1' ADDRESS='/Event' "
    + "DATAFILE='@{outputData}' TYP='POOL_ROOTTREE' OPT='RECREATE'\""
)
opt_dav = opt_dav + ';EventPersistencySvc.CnvServices += { "LHCb::RawDataCnvSvc" }'
opt_dav = opt_dav + ';ApplicationMgr.TopAlg += {"StoreExplorerAlg"}'
opt_dav = opt_dav + ";StoreExplorerAlg.Load = 1"
opt_dav = opt_dav + ";StoreExplorerAlg.PrintFreq = 0.99"
opt_dav = opt_dav + ";StoreExplorerAlg.AccessForeign = true"

# etcf = "SETC_@{STEP_ID}.root"
opt_brunel = '#include "$BRUNELOPTS/SuppressWarnings.opts"'
opt_brunel = (
    opt_brunel + ";MessageSvc.Format = '%u % F%18W%S%7W%R%T %0W%M';" + "MessageSvc.timeFormat = '%Y-%m-%d %H:%M:%S UTC'"
)
opt_brunel = opt_brunel + ";EventLoopMgr.OutputLevel = 3"
opt_brunel = opt_brunel + ";DstWriter.Output = \"DATAFILE='PFN:@{outputData}' TYP='POOL_ROOTTREE' OPT='RECREATE'\""
opt_brunel = (
    opt_brunel + ";EvtTupleSvc.Output = {\"EVTTAGS2 DATAFILE='PFN:@{etcf}' TYP='POOL_ROOTTREE' OPT='RECREATE'\"}"
)
# opt_brunel = opt_brunel+";EventSelector.Input = {\"COLLECTION=\'TagCreator/1\' DATAFILE=\'@{InputData}\' TYPE=\'POOL_ROOTTREE\' SEL=\'(GlobalOr>=1)\' OPT=\'READ\'\"}"
opt_brunel = opt_brunel + ';EventPersistencySvc.CnvServices += { "LHCb::RawDataCnvSvc" }'
opt_brunel = opt_brunel + ';ApplicationMgr.TopAlg += {"StoreExplorerAlg"}'
opt_brunel = opt_brunel + ";StoreExplorerAlg.Load = 1"
opt_brunel = opt_brunel + ";StoreExplorerAlg.PrintFreq = 0.99"
opt_brunel = opt_brunel + ";IODataManager.AgeLimit = 2"

step3 = StepDefinition("Job_Finalization")
step3.addModule(module6)
moduleInstance6 = step3.createModuleInstance("JobFinalization", "module6")
step3.addParameterLinked(module6.parameters)
step3.unlink(["poolXMLCatName", "SourceData", "DataType", "CONFIG_NAME", "CONFIG_VERSION"])


##############  WORKFLOW #################################
workflow1 = Workflow(name="CCRC-strip-test")
workflow1.setDescription("Workflow of GaudiApplication")

workflow1.addStep(step1)
step1_prefix = "step1_"
stepInstance1 = workflow1.createStepInstance("Gaudi_App_Step", "Step1")
# lets link all parameters them up with the level of workflow
stepInstance1.linkUp(stepInstance1.parameters, step1_prefix)
stepInstance1.setLink("systemConfig", "self", "SystemConfig")  # capital letter corrected
stepInstance1.unlink(
    [
        "listoutput",
        "STEP_ID",
        "optionsFile",
        "optionsLine",
        "appLog",
        "appName",
        "etcf",
        "appType",
        "outputData",
        "EVENTTYPE",
    ]
)
stepInstance1.setValue("appName", "DaVinci")
stepInstance1.setValue("appType", "root")
stepInstance1.setValue("outputData", "@{PRODUCTION_ID}_@{JOB_ID}_@{STEP_NUMBER}.@{appType}")
stepInstance1.setValue("optionsFile", "DVOfficialStrippingFile.opts")
stepInstance1.setValue("optionsLine", opt_dav)
stepInstance1.linkUp("CONFIG_NAME")
stepInstance1.linkUp("CONFIG_VERSION")
stepInstance1.linkUp("DataType")
stepInstance1.linkUp("SourceData")
stepInstance1.linkUp("poolXMLCatName")
stepInstance1.setLink("inputData", "self", "InputData")  # KGG linked with InputData of the Workflow
list1_out = [
    {
        "outputDataName": "@{PRODUCTION_ID}_@{JOB_ID}_@{STEP_NUMBER}.@{appType}",
        "outputType": "FETC",
        "outputDataSE": "Tier1_M-DST",
    }
]
stepInstance1.setValue("listoutput", list1_out)

step2_prefix = "step2_"
stepInstance2 = workflow1.createStepInstance("Gaudi_App_Step", "Step2")
# lets link all parameters them up with the level of workflow
stepInstance2.linkUp(stepInstance2.parameters, step2_prefix)
stepInstance2.setLink("systemConfig", "self", "SystemConfig")  # capital letter corrected
# except "STEP_ID", "appLog"
stepInstance2.unlink(
    [
        "listoutput",
        "STEP_ID",
        "optionsFile",
        "outputData",
        "optionsLine",
        "appLog",
        "appName",
        "appType",
        "etcf",
        "EVENTTYPE",
    ]
)
stepInstance2.setValue("appName", "Brunel")
stepInstance2.setValue("appType", "dst")
stepInstance2.setValue("outputData", "@{PRODUCTION_ID}_@{JOB_ID}_@{STEP_NUMBER}.@{appType}")
stepInstance2.setValue("etcf", "SETC_@{PRODUCTION_ID}_@{JOB_ID}_@{STEP_NUMBER}.root")
stepInstance2.setValue("optionsFile", "RealData-ETC.opts")
stepInstance2.setValue("optionsLine", opt_brunel)
# stepInstance2.setValue("outputDataSE","Tier1_M-DST")
stepInstance2.linkUp("CONFIG_NAME")
stepInstance2.linkUp("CONFIG_VERSION")
stepInstance2.linkUp("DataType")
stepInstance2.linkUp("SourceData")
stepInstance2.linkUp("poolXMLCatName")
stepInstance2.setLink("inputData", stepInstance1.getName(), "outputData")
list2_out = [
    {
        "outputDataName": "@{PRODUCTION_ID}_@{JOB_ID}_@{STEP_NUMBER}.@{appType}",
        "outputType": "dst",
        "outputDataSE": "Tier1_M-DST",
    },
    {
        "outputDataName": "SETC_@{PRODUCTION_ID}_@{JOB_ID}_@{STEP_NUMBER}.root",
        "outputType": "SETC",
        "outputDataSE": "Tier1_M-DST",
    },
]
stepInstance2.setValue("listoutput", list2_out)

workflow1.addStep(step3)
step3_prefix = "step3_"
stepInstance3 = workflow1.createStepInstance("Job_Finalization", "Step3")
# stepInstance3.linkUp(stepInstance3.parameters, step3_prefix)
stepInstance3.linkUp("CONFIG_NAME")
stepInstance3.linkUp("CONFIG_VERSION")
stepInstance3.linkUp("DataType")
stepInstance3.linkUp("SourceData")
stepInstance3.linkUp("poolXMLCatName")
# I am adding
stepInstance3.unlink(["listoutput_1", "listoutput_2", "listoutput_3", "listoutput_4", "listoutput_5", "listoutput_6"])
stepInstance3.setLink("listoutput_1", stepInstance1.getName(), "listoutput")
stepInstance3.setLink("listoutput_2", stepInstance2.getName(), "listoutput")
# **************************
workflow1.addParameterLinked(step1.parameters, step1_prefix)
workflow1.addParameterLinked(step1.parameters, step2_prefix)
# and finally we can unlink them because we inherit them linked
workflow1.unlink(workflow1.parameters)

workflow1.setValue(step1_prefix + "appVersion", "v19r11")
workflow1.setValue(step2_prefix + "appVersion", "v32r4")
workflow1.setValue(step1_prefix + "NUMBER_OF_EVENTS", "5")
workflow1.setValue(step2_prefix + "NUMBER_OF_EVENTS", "-1")
workflow1.removeParameter(step1_prefix + "inputData")  # KGG wrong parameter
workflow1.setValue(step1_prefix + "inputDataType", "RDST")
workflow1.setValue(step2_prefix + "inputDataType", "ETC")
workflow1.setValue(step1_prefix + "OUTPUT_MAX", "20")
# remove unwanted
workflow1.removeParameter(step1_prefix + "outputData")
workflow1.removeParameter(step1_prefix + "systemConfig")
workflow1.removeParameter(step2_prefix + "outputData")
workflow1.removeParameter(step2_prefix + "systemConfig")

# add syspem config which common for all modules
# workflow1.addParameter(Parameter("systemConfig","slc4_ia32_gcc34","string","","",True, False, "Application Name"))
workflow1.addParameter(
    Parameter("SystemConfig", "x86_64-slc5-gcc43-opt", "JDLReqt", "", "", True, False, "Application Name")
)

# Now lets define parameters on the top
# indata = "LFN:/lhcb/production/DC06/phys-v2-lumi2/00001820/SIM/0000/00001820_00000001_1.sim;LFN:/lhcb/production/DC06/phys-v2-lumi2/00001820/SIM/0000/00001820_00000001_2.sim;LFN:/lhcb/production/DC06/phys-v2-lumi2/00001820/SIM/0000/00001820_00000001_3.sim"
indata = "LFN:/lhcb/data/CCRC08/RDST/00000106/0000/00000106_00007918_1.rdst"
# indata = "LFN:/lhcb/data/CCRC08/RDST/00000106/0000/00000106_00007918_1.rdst;/lhcb/data/CCRC08/RAW/LHCb/CCRC/420217/420217_0000116193.raw"
# etcf = "joel.root"
# indata = "LFN:/lhcb/data/CCRC08/RAW/LHCb/CCRC/402154/402154_0000047096.raw;LFN:/lhcb/data/CCRC08/RAW/LHCb/CCRC/402154/402154_0000047097.raw"
# lets specify parameters on the level of workflow


workflow1.addParameter(Parameter("InputData", indata, "JDL", "", "", True, False, "Application Name"))
workflow1.addParameter(Parameter("JobType", "test", "JDL", "", "", True, False, "Job TYpe"))
workflow1.addParameter(Parameter("AncestorDepth", "2", "JDL", "", "", True, False, "Ancestor Depth"))
workflow1.addParameter(Parameter("Owner", "joel", "JDL", "", "", True, False, "user Name"))
workflow1.addParameter(Parameter("StdError", "std.err", "JDL", "", "", True, False, "user Name"))
workflow1.addParameter(Parameter("StdOutput", "std.out", "JDL", "", "", True, False, "user Name"))
workflow1.addParameter(Parameter("SoftwarePackages", "Brunel.v32r4", "JDL", "", "", True, False, "software"))

workflow1.addParameter(Parameter("CPUTime", 300000, "JDLReqt", "", "", True, False, "Application Name"))
# workflow1.addParameter(Parameter("Site","LCG.CERN.ch","JDLReqt","","",True, False, "Site"))
workflow1.addParameter(Parameter("Platform", "gLite", "JDLReqt", "", "", True, False, "platform"))

# and finally we can unlink them because we inherit them linked
workflow1.unlink(workflow1.parameters)

workflow1.addParameter(Parameter("PRODUCTION_ID", "00003033", "string", "", "", True, False, "Temporary fix"))
workflow1.addParameter(Parameter("JOB_ID", "00000011", "string", "", "", True, False, "Temporary fix"))
workflow1.addParameter(
    Parameter(
        "EMAILNAME",
        "joel.closier@cern.ch",
        "string",
        "",
        "",
        True,
        False,
        "Email to send a report from the LogCheck module",
    )
)
workflow1.addParameter(Parameter("DataType", "DATA", "string", "", "", True, False, "type of Datatype"))
workflow1.addParameter(Parameter("SourceData", indata, "string", "", "", True, False, "Application Name"))
workflow1.addParameter(
    Parameter("poolXMLCatName", "pool_xml_catalog.xml", "string", "", "", True, False, "Application Name")
)
workflow1.addParameter(Parameter("CONFIG_NAME", "LHCb", "string", "", "", True, False, "Configuration Name"))
workflow1.addParameter(Parameter("CONFIG_VERSION", "CCRC08", "string", "", "", True, False, "Configuration Version"))
# workflow1.addParameter(Parameter("NUMBER_OF_EVENTS","5","string","","",True, False, "number of events requested"))
workflow1.toXMLFile("wkf_CCRC_3.xml")
# w4 = fromXMLFile("/afs/cern.ch/user/g/gkuznets/test1.xml")
# print 'Creating code for the workflow'
# eval(compile(workflow1.createCode(),'<string>','exec'))
# workflow1.execute()



########################################################################
# File :    dirac-proxy-init.py
# Author :  Adrian Casajus
########################################################################

from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
import sys
from prompt_toolkit import prompt
import DIRAC

from DIRAC import gLogger
from DIRAC.Core.Utilities.DIRACScript import DIRACScript as Script

__RCSID__ = "$Id$"


class CLIParams(object):

    proxyLifeTime = 2592000
    certLoc = False
    keyLoc = False
    proxyLoc = False
    onTheFly = False
    stdinPasswd = False
    rfcIfPossible = False
    userPasswd = ""

    def __str__(self):
        data = []
        for k in ("proxyLifeTime", "certLoc", "keyLoc", "proxyLoc", "onTheFly", "stdinPasswd", "userPasswd"):
            if k == "userPasswd":
                data.append("userPasswd = *****")
            else:
                data.append("%s=%s" % (k, getattr(self, k)))
        msg = "<UploadCLIParams %s>" % " ".join(data)
        return msg

    def setProxyLifeTime(self, arg):
        try:
            fields = [f.strip() for f in arg.split(":")]
            self.proxyLifeTime = int(fields[0]) * 3600 + int(fields[1]) * 60
        except ValueError:
            gLogger.notice("Can't parse %s time! Is it a HH:MM?" % arg)
            return DIRAC.S_ERROR("Can't parse time argument")
        return DIRAC.S_OK()

    def setProxyRemainingSecs(self, arg):
        self.proxyLifeTime = int(arg)
        return DIRAC.S_OK()

    def getProxyLifeTime(self):
        hours = int(self.proxyLifeTime / 3600)
        mins = int(self.proxyLifeTime / 60 - hours * 60)
        return "%s:%s" % (hours, mins)

    def getProxyRemainingSecs(self):
        return self.proxyLifeTime

    def setCertLocation(self, arg):
        self.certLoc = arg
        return DIRAC.S_OK()

    def setKeyLocation(self, arg):
        self.keyLoc = arg
        return DIRAC.S_OK()

    def setProxyLocation(self, arg):
        self.proxyLoc = arg
        return DIRAC.S_OK()

    def setOnTheFly(self, arg):
        self.onTheFly = True
        return DIRAC.S_OK()

    def setStdinPasswd(self, arg):
        self.stdinPasswd = True
        return DIRAC.S_OK()

    def showVersion(self, arg):
        gLogger.notice("Version:")
        gLogger.notice(" ", __RCSID__)
        sys.exit(0)
        return DIRAC.S_OK()

    def registerCLISwitches(self):
        Script.registerSwitch(
            "v:", "valid=", "Valid HH:MM for the proxy. By default is one month", self.setProxyLifeTime
        )
        Script.registerSwitch("C:", "Cert=", "File to use as user certificate", self.setCertLocation)
        Script.registerSwitch("K:", "Key=", "File to use as user key", self.setKeyLocation)
        Script.registerSwitch("P:", "Proxy=", "File to use as proxy", self.setProxyLocation)
        Script.registerSwitch("f", "onthefly", "Generate a proxy on the fly", self.setOnTheFly)
        Script.registerSwitch("p", "pwstdin", "Get passwd from stdin", self.setStdinPasswd)
        Script.registerSwitch("i", "version", "Print version", self.showVersion)
        Script.addDefaultOptionValue("LogLevel", "always")


from DIRAC import S_ERROR
from DIRAC.Core.Security import Locations
from DIRAC.Core.Security.X509Chain import X509Chain  # pylint: disable=import-error
from DIRAC.FrameworkSystem.Client.ProxyManagerClient import gProxyManager


def uploadProxy(params):
    DIRAC.gLogger.info("Loading user proxy")
    proxyLoc = params.proxyLoc
    if not proxyLoc:
        proxyLoc = Locations.getDefaultProxyLocation()
    if not proxyLoc:
        return S_ERROR("Can't find any proxy")

    if params.onTheFly:
        DIRAC.gLogger.info("Uploading proxy on-the-fly")
        certLoc = params.certLoc
        keyLoc = params.keyLoc
        if not certLoc or not keyLoc:
            cakLoc = Locations.getCertificateAndKeyLocation()
            if not cakLoc:
                return S_ERROR("Can't find user certificate and key")
            if not certLoc:
                certLoc = cakLoc[0]
            if not keyLoc:
                keyLoc = cakLoc[1]

        DIRAC.gLogger.info("Cert file %s" % certLoc)
        DIRAC.gLogger.info("Key file  %s" % keyLoc)

        testChain = X509Chain()
        retVal = testChain.loadKeyFromFile(keyLoc, password=params.userPasswd)
        if not retVal["OK"]:
            if params.stdinPasswd:
                userPasswd = sys.stdin.readline().strip("\n")
            else:
                try:
                    userPasswd = prompt(u"Enter Certificate password: ", is_password=True)
                except KeyboardInterrupt:
                    return S_ERROR("Caught KeyboardInterrupt, exiting...")
            params.userPasswd = userPasswd

        DIRAC.gLogger.info("Loading cert and key")
        chain = X509Chain()
        # Load user cert and key
        retVal = chain.loadChainFromFile(certLoc)
        if not retVal["OK"]:
            return S_ERROR("Can't load %s" % certLoc)
        retVal = chain.loadKeyFromFile(keyLoc, password=params.userPasswd)
        if not retVal["OK"]:
            return S_ERROR("Can't load %s" % keyLoc)
        DIRAC.gLogger.info("User credentials loaded")
        restrictLifeTime = params.proxyLifeTime

    else:
        proxyChain = X509Chain()
        retVal = proxyChain.loadProxyFromFile(proxyLoc)
        if not retVal["OK"]:
            return S_ERROR("Can't load proxy file %s: %s" % (params.proxyLoc, retVal["Message"]))

        chain = proxyChain
        restrictLifeTime = 0

    DIRAC.gLogger.info(" Uploading...")
    return gProxyManager.uploadProxy(proxy=chain, restrictLifeTime=restrictLifeTime, rfcIfPossible=params.rfcIfPossible)



"""
  Defines the plugin to perform evaluation on the user in the proxy
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

__RCSID__ = "$Id $"

import re


from DIRAC.Resources.Catalog.ConditionPlugins.FCConditionBasePlugin import FCConditionBasePlugin
from DIRAC.Core.Security.ProxyInfo import getProxyInfo


class ProxyPlugin(FCConditionBasePlugin):
    """
    This plugin is to be used when filtering based on the user contained in the proxy
    """

    def __init__(self, conditions):
        """This plugin allows to perform tests on the proxy.
        Supported conditions are:

          * username.in(<comma separated list of names>): the user should be in the defined list
          * username.not_in(<comma separated list of names>): the user should *not* be in the defined list
          * group.in(<comma separated list of names>): the group should be in the defined list
          * group.not_in(<comma separated list of names>): the group should *not* be in the defined list
          * property.has(<property name>): the proxy should have the given property
          * property.has_not(<property name>): the proxy should *not* have the given property
          * voms.has(<voms role>) (see bellow): if the given VOMS role is associated to the proxy
          * voms.has_not(<voms role>)  (see bellow): if the given VOMS role is *not* associated to the proxy

        Because it is not possible to use the '=' sign, the VOMS role has to be declared using the
        symbol '->'.
        For example::

          "voms.has(/lhcb/Role->production)" will look for "/lhcb/Role=production" in the VOMS list contained
          in the proxy info.


        If the conditions does not follow the form 'attribute.predicate(value)', an exception
        is thrown, and will lead to all the expression be evaluated to False

        If there is no proxy, all conditions are evaluated to False

        """
        super(ProxyPlugin, self).__init__(conditions)

        # the conditions have the form
        # attribute.predicate(value)
        condition_pattern = r"^(\w+)\.(\w+)\((.+)\)$"
        regex = re.compile(condition_pattern)
        match = regex.search(conditions)
        self.attr, self.predicate, self.value = match.groups()

        # To cover the case where the names would be surrounded by (single) quotes
        self.value = self.value.replace("'", "").replace('"', "").replace(" ", "")

        self._checkCondition()
        self.proxyInfo = getProxyInfo().get("Value")

    def _checkCondition(self):
        """Checks that the actual condition makes sense
        if not, raises a RuntimeError exception
        """

        excp = RuntimeError("Incorrect condition format %s" % self.conditions)

        if self.attr in ["username", "group"]:
            if self.predicate not in ["in", "not_in"]:
                raise excp
        elif self.attr in ["property", "voms"]:
            if self.predicate not in ["has", "has_not"]:
                raise excp
        else:
            raise excp

    def eval(self, **kwargs):
        """evaluate the parameters."""

        if not self.proxyInfo:
            return False

        valueToLookFor = None
        listToLookInto = []

        if self.attr in ["username", "group"]:
            valueToLookFor = self.proxyInfo.get(self.attr)
            listToLookInto = self.value.split(",")
        elif self.attr == "property":
            valueToLookFor = self.value
            listToLookInto = self.proxyInfo.get("groupProperties", [])
        elif self.attr == "voms":
            valueToLookFor = self.value.replace("->", "=")
            listToLookInto = self.proxyInfo.get("VOMS", [])

        if "not" in self.predicate:
            return valueToLookFor not in listToLookInto
        else:
            return valueToLookFor in listToLookInto



""" DataIntegrityDB class is a front-end to the Data Integrity Database.
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

from DIRAC import S_OK
from DIRAC.Core.Base.DB import DB

__RCSID__ = "$Id$"

#############################################################################


class DataIntegrityDB(DB):
    """Only 1 table, that can be created with:

    .. code-block:: sql

        CREATE TABLE Problematics(
          FileID INTEGER NOT NULL AUTO_INCREMENT,
          Prognosis VARCHAR(32) NOT NULL,
          LFN VARCHAR(255) NOT NULL,
          PFN VARCHAR(255),
          Size BIGINT(20),
          SE VARCHAR(32),
          GUID VARCHAR(255),
          Status VARCHAR(32) DEFAULT 'New',
          Retries INTEGER DEFAULT 0,
          InsertDate DATETIME NOT NULL,
          LastUpdate DATETIME NOT NULL,
          Source VARCHAR(127) NOT NULL DEFAULT 'Unknown',
          PRIMARY KEY(FileID),
          INDEX (Prognosis,Status)
        );

    """

    def __init__(self):
        """Standard Constructor"""
        DB.__init__(self, "DataIntegrityDB", "DataManagement/DataIntegrityDB")

        self.tableName = "Problematics"
        self.fieldList = ["FileID", "LFN", "PFN", "Size", "SE", "GUID", "Prognosis"]

        retVal = self.__initializeDB()
        if not retVal["OK"]:
            raise Exception("Can't create tables: %s" % retVal["Message"])

    def __initializeDB(self):
        """Make sure the table is created"""
        result = self._query("show tables")
        if not result["OK"]:
            return result

        tablesInDB = [t[0] for t in result["Value"]]
        if self.tableName in tablesInDB:
            return S_OK()
        tablesDesc = {}
        tablesDesc[self.tableName] = {
            "Fields": {
                "FileID": "INTEGER NOT NULL AUTO_INCREMENT",
                "Prognosis": "VARCHAR(32) NOT NULL",
                "LFN": "VARCHAR(255) NOT NULL",
                "PFN": "VARCHAR(255)",
                "Size": "BIGINT(20)",
                "SE": "VARCHAR(32)",
                "GUID": "VARCHAR(255)",
                "Status": 'VARCHAR(32) DEFAULT "New"',
                "Retries": "INTEGER DEFAULT 0",
                "InsertDate": "DATETIME NOT NULL",
                "LastUpdate": "DATETIME NOT NULL",
                "Source": 'VARCHAR(127) NOT NULL DEFAULT "Unknown"',
            },
            "PrimaryKey": "FileID",
            "Indexes": {"PS": ["Prognosis", "Status"]},
            "Engine": "InnoDB",
        }

        return self._createTables(tablesDesc)

    #############################################################################
    def insertProblematic(self, source, fileMetadata):
        """Insert the supplied file metadata into the problematics table"""
        failed = {}
        successful = {}
        for lfn, metadata in fileMetadata.items():
            condDict = dict((key, metadata[key]) for key in ["Prognosis", "PFN", "SE"])
            condDict["LFN"] = lfn
            res = self.getFields(self.tableName, ["FileID"], condDict=condDict)
            if not res["OK"]:
                failed[lfn] = res["Message"]
            elif res["Value"]:
                successful[lfn] = "Already exists"
            else:
                metadata["LFN"] = lfn
                metadata["Source"] = source
                metadata["InsertDate"] = "UTC_TIMESTAMP()"
                metadata["LastUpdate"] = "UTC_TIMESTAMP()"
                res = self.insertFields(self.tableName, inDict=metadata)
                if res["OK"]:
                    successful[lfn] = True
                else:
                    failed[lfn] = res["Message"]
        resDict = {"Successful": successful, "Failed": failed}
        return S_OK(resDict)

    #############################################################################
    def getProblematicsSummary(self):
        """Get a summary of the current problematics table"""
        res = self.getCounters(self.tableName, ["Prognosis", "Status"], {})
        if not res["OK"]:
            return res
        resDict = {}
        for counterDict, count in res["Value"]:
            resDict.setdefault(counterDict["Prognosis"], {})
            resDict[counterDict["Prognosis"]][counterDict["Status"]] = int(count)
        return S_OK(resDict)

    #############################################################################
    def getDistinctPrognosis(self):
        """Get a list of all the current problematic types"""
        return self.getDistinctAttributeValues(self.tableName, "Prognosis")

    #############################################################################
    def getProblematic(self):
        """Get the next file to resolve"""
        res = self.getFields(
            self.tableName, self.fieldList, condDict={"Status": "New"}, limit=1, orderAttribute="LastUpdate:ASC"
        )
        if not res["OK"]:
            return res
        if not res["Value"][0]:
            return S_OK()
        valueList = list(res["Value"][0])
        return S_OK(dict((key, valueList.pop(0)) for key in self.fieldList))

    def getPrognosisProblematics(self, prognosis):
        """Get all the active files with the given problematic"""
        res = self.getFields(
            self.tableName,
            self.fieldList,
            condDict={"Prognosis": prognosis, "Status": "New"},
            orderAttribute=["Retries", "LastUpdate"],
        )
        if not res["OK"]:
            return res
        problematics = []
        for valueTuple in res["Value"]:
            valueList = list(valueTuple)
            problematics.append(dict((key, valueList.pop(0)) for key in self.fieldList))
        return S_OK(problematics)

    def getTransformationProblematics(self, transID):
        """Get problematic files matching a given production"""
        req = "SELECT LFN,FileID FROM Problematics WHERE Status = 'New' AND LFN LIKE '%%/%08d/%%';" % transID
        res = self._query(req)
        if not res["OK"]:
            return res
        problematics = {}
        for lfn, fileID in res["Value"]:
            problematics[lfn] = fileID
        return S_OK(problematics)

    def incrementProblematicRetry(self, fileID):
        """Increment retry count"""
        req = "UPDATE Problematics SET Retries=Retries+1, LastUpdate=UTC_TIMESTAMP() WHERE FileID = %s;" % (fileID)
        res = self._update(req)
        return res

    def removeProblematic(self, fileID):
        """Remove Problematic file by FileID"""
        return self.deleteEntries(self.tableName, condDict={"FileID": fileID})

    def setProblematicStatus(self, fileID, status):
        """Set Status for problematic file by FileID"""
        return self.updateFields(
            self.tableName, condDict={"FileID": fileID}, updateDict={"Status": status, "LastUpdate": "UTC_TIMESTAMP()"}
        )

    def changeProblematicPrognosis(self, fileID, newPrognosis):
        """Change prognisis for file by FileID"""
        return self.updateFields(
            self.tableName,
            condDict={"FileID": fileID},
            updateDict={"Prognosis": newPrognosis, "LastUpdate": "UTC_TIMESTAMP()"},
        )



""" DIRAC.ResourceStatusSystem.Policy.PropagationPolicy

    The following lines are needed in the CS::

      PropagationPolicy
      {
        matchParams
        {
          element = Site
        }
        policyType = PropagationPolicy
      }

"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

from DIRAC import S_OK
from DIRAC.ResourceStatusSystem.PolicySystem.PolicyBase import PolicyBase

__RCSID__ = "$Id$"


class PropagationPolicy(PolicyBase):
    """
    PropagationPolicy module doc
    """

    @staticmethod
    def _evaluate(commandResult):
        """
        commandResult is the result of 'PropagationCommand' which
        indicates if a site should be 'Active' or 'Banned'

        :returns:
           {
           `Status`:Error|Unknown|Active|Banned,
           `Reason`:'A:X/P:Y/B:Z'
           }
        """

        result = {"Status": None, "Reason": None}

        if not commandResult["OK"]:

            result["Status"] = "Error"
            result["Reason"] = commandResult["Message"]
            return S_OK(result)

        else:

            commandResult = commandResult["Value"]

            result["Status"] = commandResult["Status"]
            result["Reason"] = commandResult["Reason"]
            return S_OK(result)



"""Tests for the ArchiveFiles Operation"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

# pylint: disable=protected-access, redefined-outer-name

import six
from functools import partial
import logging
import os

import pytest
from mock import MagicMock as Mock

from DIRAC import S_ERROR, S_OK
from DIRAC.Core.Utilities import DEncode
from DIRAC.RequestManagementSystem.Client.File import File
from DIRAC.RequestManagementSystem.Client.Operation import Operation
from DIRAC.RequestManagementSystem.Client.Request import Request

from DIRAC.DataManagementSystem.Agent.RequestOperations import ArchiveFiles

logging.basicConfig(level=logging.WARNING, format="%(levelname)-5s - %(name)-8s: %(message)s")
LOG = logging.getLogger("TestArchiveFiles")

MODULE = "DIRAC.DataManagementSystem.Agent.RequestOperations.ArchiveFiles"
FILE_NAME = "fileName"
N_FILES = 10
DEST_DIR = "/Some/Local/Folder"


@pytest.fixture
def listOfLFNs():
    """Return a list of LFNs"""
    lfns = []
    for index, name in enumerate([FILE_NAME] * N_FILES):
        lfns.append("/vo/%s_%d" % (name, index))
    return lfns


@pytest.fixture
def _myMocker(mocker):
    """Mock call to external libraries."""
    mocker.patch(MODULE + ".shutil.make_archive")
    mocker.patch(MODULE + ".shutil.rmtree")
    mocker.patch(MODULE + ".os.makedirs")
    mocker.patch(MODULE + ".os.remove")
    mocker.patch(MODULE + ".gMonitor")
    return None


@pytest.fixture
def multiRetValOK(listOfLFNs):
    """Return a return structure for multiple values"""
    retVal = {
        "OK": True,
        "Value": {
            "Failed": {},
            "Successful": {},
        },
    }
    for lfn in listOfLFNs:
        retVal["Value"]["Successful"][lfn] = True
    return retVal


def multiRetVal(*args, **kwargs):
    """Return a return structure for multiple values"""
    retVal = {
        "OK": True,
        "Value": {
            "Failed": {},
            "Successful": {},
        },
    }
    lfns = args[0]
    if isinstance(lfns, six.string_types):
        lfns = [lfns]
    for _index, lfn in enumerate(lfns):
        if str(kwargs.get("Index", 5)) in lfn:
            retVal["Value"]["Failed"][lfn] = kwargs.get("Error", "Failed to do X")
            LOG.error("Error for %s %s", lfn, retVal["Value"]["Failed"][lfn])
        else:
            retVal["Value"]["Successful"][lfn] = kwargs.get("Success", True)
            LOG.info("Success for %s %s", lfn, retVal["Value"]["Successful"])
    return retVal


@pytest.fixture
def archiveRequestAndOp(listOfLFNs):
    """Return a tuple of the request and operation."""
    req = Request()
    req.RequestName = "MyRequest"
    op = Operation()
    switches = {}
    archiveLFN = "/vo/tars/myTar.tar"
    op.Arguments = DEncode.encode(
        {
            "SourceSE": switches.get("SourceSE", "SOURCE-SE"),
            "TarballSE": switches.get("TarballSE", "TARBALL-SE"),
            "RegisterDescendent": False,
            "ArchiveLFN": archiveLFN,
        }
    )
    op.Type = "ArchiveFiles"
    for index, lfn in enumerate(listOfLFNs):
        oFile = File()
        oFile.LFN = lfn
        oFile.Size = index
        oFile.Checksum = "01130a%0d" % index
        oFile.ChecksumType = "adler32"
        op.addFile(oFile)

    req.addOperation(op)
    return req, op


@pytest.fixture
def archiveFiles(mocker, archiveRequestAndOp, multiRetValOK):
    """Return the ArchiveFiles operation instance."""
    mocker.patch.dict(os.environ, {"AGENT_WORKDIRECTORY": DEST_DIR})
    af = ArchiveFiles.ArchiveFiles(archiveRequestAndOp[1])
    af.fc = mocker.MagicMock("FileCatalogMock")
    af.fc.hasAccess = mocker.MagicMock()
    af.fc.hasAccess.return_value = multiRetValOK
    af.fc.getReplicas = mocker.MagicMock()
    af.fc.getReplicas.side_effect = partial(multiRetVal, Success={"SOURCE-SE": "PFN"}, Index=11)
    af.fc.isFile = mocker.MagicMock()
    archiveLFN = "/vo/tars/myTar.tar"
    af.fc.isFile.return_value = S_OK({"Failed": {archiveLFN: "no file"}, "Successful": {}})
    af.dm = mocker.MagicMock("DataManagerMock")
    af.dm.getFile = mocker.MagicMock(return_value=multiRetValOK)
    af.dm.putAndRegister = mocker.MagicMock(return_value=multiRetValOK)
    return af


def test_constructor(archiveFiles):
    assert archiveFiles.parameterDict == {}
    assert archiveFiles.lfns == []
    assert archiveFiles.waitingFiles == []
    assert archiveFiles.cacheFolder == "/Some/Local/Folder"


def test_run_OK(archiveFiles, _myMocker, listOfLFNs):
    archiveFiles._run()
    archiveFiles.dm.getFile.assert_called_with(
        listOfLFNs[9], destinationDir=os.path.join(DEST_DIR, "MyRequest", "vo"), sourceSE="SOURCE-SE"
    )
    for opFile in archiveFiles.operation:
        assert opFile.Status == "Done"


def test_run_Fail(archiveFiles, _myMocker, listOfLFNs):
    archiveFiles.dm.getFile.side_effect = partial(multiRetVal, Index=5)
    with pytest.raises(RuntimeError, match="Completely failed to download file"):
        archiveFiles._run()
    archiveFiles.dm.getFile.assert_called_with(
        listOfLFNs[5], destinationDir=os.path.join(DEST_DIR, "MyRequest", "vo"), sourceSE="SOURCE-SE"
    )
    for opFile in archiveFiles.operation:
        assert opFile.Status == "Waiting"


def test_run_IgnoreMissingFiles(archiveFiles, _myMocker, listOfLFNs):
    archiveFiles.dm.getFile.side_effect = partial(multiRetVal, Index=5, Error="No such file or directory")
    archiveFiles._run()
    archiveFiles.dm.getFile.assert_called_with(
        listOfLFNs[9], destinationDir=os.path.join(DEST_DIR, "MyRequest", "vo"), sourceSE="SOURCE-SE"
    )
    for index, opFile in enumerate(archiveFiles.operation):
        LOG.debug("%s", opFile)  # lazy evaluation of the argument!
        if index == 5:
            assert opFile.Status == "Done"
        else:
            assert opFile.Status == "Done"


def test_checkFilePermissions(archiveFiles, _myMocker):
    archiveFiles.waitingFiles = archiveFiles.getWaitingFilesList()
    archiveFiles.lfns = [opFile.LFN for opFile in archiveFiles.waitingFiles]
    assert len(archiveFiles.lfns) == N_FILES
    archiveFiles.fc.hasAccess.side_effect = partial(multiRetVal, Index=3, Error="Permission denied")
    with pytest.raises(RuntimeError, match="^Do not have sufficient permissions$"):
        archiveFiles._checkFilePermissions()
    for index, opFile in enumerate(archiveFiles.operation):
        if index == 3:
            assert opFile.Status == "Failed"
        else:
            assert opFile.Status == "Waiting"


def test_checkFilePermissions_breaks(archiveFiles, _myMocker):
    archiveFiles.waitingFiles = archiveFiles.getWaitingFilesList()
    archiveFiles.lfns = [opFile.LFN for opFile in archiveFiles.waitingFiles]
    assert len(archiveFiles.lfns) == N_FILES
    archiveFiles.fc.hasAccess.return_value = S_ERROR("Break")
    with pytest.raises(RuntimeError, match="^Could not resolve permissions$"):
        archiveFiles._checkFilePermissions()
    for opFile in archiveFiles.operation:
        assert opFile.Status == "Waiting"


def test_uploadTarBall_breaks(archiveFiles, _myMocker, listOfLFNs):
    archiveFiles.dm.putAndRegister.return_value = S_ERROR("Break")
    with pytest.raises(RuntimeError, match="^Failed to upload tarball: Break$"):
        archiveFiles._run()
    for opFile in archiveFiles.operation:
        assert opFile.Status == "Waiting"
    archiveFiles.dm.getFile.assert_called_with(
        listOfLFNs[9], destinationDir=os.path.join(DEST_DIR, "MyRequest", "vo"), sourceSE="SOURCE-SE"
    )
    archiveFiles.dm.putAndRegister.assert_called_with("/vo/tars/myTar.tar", "myTar.tar", "TARBALL-SE")


def test_call(archiveFiles, _myMocker, listOfLFNs):
    archiveFiles()
    for opFile in archiveFiles.operation:
        assert opFile.Status == "Done"
    archiveFiles.dm.getFile.assert_called_with(
        listOfLFNs[9], destinationDir=os.path.join(DEST_DIR, "MyRequest", "vo"), sourceSE="SOURCE-SE"
    )
    archiveFiles.dm.putAndRegister.assert_called_with("/vo/tars/myTar.tar", "myTar.tar", "TARBALL-SE")


def test_call_withError(archiveFiles, _myMocker, listOfLFNs):
    archiveFiles.dm.putAndRegister.return_value = S_ERROR("Break")
    archiveFiles()
    for opFile in archiveFiles.operation:
        assert opFile.Status == "Waiting"
    archiveFiles.dm.getFile.assert_called_with(
        listOfLFNs[9], destinationDir=os.path.join(DEST_DIR, "MyRequest", "vo"), sourceSE="SOURCE-SE"
    )
    archiveFiles.dm.putAndRegister.assert_called_with("/vo/tars/myTar.tar", "myTar.tar", "TARBALL-SE")


def test_call_withUnexpectedError(archiveFiles, _myMocker):
    archiveFiles.operation.Arguments = ""
    res = archiveFiles()
    assert not res["OK"]


def test_cleanup(archiveFiles, mocker):
    osMocker = mocker.patch(MODULE + ".os.remove", side_effect=OSError("No such file or directory"))
    rmTreeMock = mocker.patch(MODULE + ".shutil.rmtree", side_effect=OSError("No such file or directory"))
    archiveFiles.parameterDict = {"ArchiveLFN": "/vo.lfn/nofile.tar"}
    archiveFiles._cleanup()
    osMocker.assert_called_with("nofile.tar")
    rmTreeMock.assert_called_with(archiveFiles.cacheFolder, ignore_errors=True)


def test_checkArchiveLFN(archiveFiles):
    archiveLFN = "/vo/tars/myTar.tar"
    archiveFiles.parameterDict = {"ArchiveLFN": archiveLFN}
    # tarball does not exist
    archiveFiles._checkArchiveLFN()
    archiveFiles.fc.isFile.assert_called_with(archiveLFN)


def test_checkArchiveLFN_Fail(archiveFiles):
    archiveLFN = "/vo/tars/myTar.tar"
    archiveFiles.parameterDict = {"ArchiveLFN": archiveLFN}
    # tarball already exists
    archiveFiles.fc.isFile.side_effect = multiRetVal
    with pytest.raises(RuntimeError, match="already exists$"):
        archiveFiles._checkArchiveLFN()


def test_checkReplicas_success(archiveFiles):
    archiveFiles.waitingFiles = archiveFiles.getWaitingFilesList()
    archiveFiles.parameterDict = {"SourceSE": "SOURCE-SE"}
    archiveFiles.lfns = [opFile.LFN for opFile in archiveFiles.waitingFiles]
    archiveFiles.fc.getReplicas.side_effect = partial(multiRetVal, Index=11, Success={"SOURCE-SE": "PFN"})
    assert archiveFiles._checkReplicas() is None


def test_checkReplicas_notAt(archiveFiles):
    archiveFiles.waitingFiles = archiveFiles.getWaitingFilesList()
    archiveFiles.parameterDict = {"SourceSE": "SOURCE-SE"}
    archiveFiles.lfns = [opFile.LFN for opFile in archiveFiles.waitingFiles]
    archiveFiles.fc.getReplicas.side_effect = partial(multiRetVal, Index=11, Success={"Not-SOURCE-SE": "PFN"})
    with pytest.raises(RuntimeError, match="Some replicas are not at the source"):
        archiveFiles._checkReplicas()


def test_checkReplicas_noSuchFile(archiveFiles):
    archiveFiles.waitingFiles = archiveFiles.getWaitingFilesList()
    archiveFiles.parameterDict = {"SourceSE": "SOURCE-SE"}
    archiveFiles.lfns = [opFile.LFN for opFile in archiveFiles.waitingFiles]
    archiveFiles.fc.getReplicas.side_effect = partial(
        multiRetVal, Index=7, Success={"SOURCE-SE": "PFN"}, Error="No such file or directory"
    )
    assert archiveFiles._checkReplicas() is None


def test_checkReplicas_somefailed(archiveFiles):
    archiveFiles.waitingFiles = archiveFiles.getWaitingFilesList()
    archiveFiles.parameterDict = {"SourceSE": "SOURCE-SE"}
    archiveFiles.lfns = [opFile.LFN for opFile in archiveFiles.waitingFiles]
    archiveFiles.fc.getReplicas.side_effect = partial(
        multiRetVal, Index=7, Success={"SOURCE-SE": "PFN"}, Error="some error"
    )
    with pytest.raises(RuntimeError, match="Failed to get some replica information"):
        archiveFiles._checkReplicas()


def test_checkReplicas_failed(archiveFiles, mocker):
    archiveFiles.waitingFiles = archiveFiles.getWaitingFilesList()
    archiveFiles.parameterDict = {"SourceSE": "SOURCE-SE"}
    archiveFiles.lfns = [opFile.LFN for opFile in archiveFiles.waitingFiles]
    archiveFiles.fc.getReplicas = mocker.MagicMock()
    archiveFiles.fc.getReplicas.return_value = S_ERROR("some error")
    with pytest.raises(RuntimeError, match="Failed to get replica information"):
        archiveFiles._checkReplicas()


def test_registerDescendent_disabled(archiveFiles):
    archiveFiles.parameterDict = {"RegisterDescendent": False}
    archiveFiles.lfns = [opFile.LFN for opFile in archiveFiles.waitingFiles]
    archiveFiles.fc.addFileAncestors = Mock(name="AFA")
    assert archiveFiles._registerDescendent() is None
    archiveFiles.fc.addFileAncestors.assert_not_called()


def test_registerDescendent_success(archiveFiles, listOfLFNs):
    archiveFiles.lfns = listOfLFNs
    archiveLFN = "/vo/tars/myTar.tar"
    archiveFiles.parameterDict = {"RegisterDescendent": True, "ArchiveLFN": archiveLFN}
    archiveFiles.fc.addFileAncestors = Mock(
        name="AFA", return_value=S_OK({"Failed": {}, "Successful": {archiveLFN: "Done"}})
    )
    assert archiveFiles._registerDescendent() is None
    archiveFiles.fc.addFileAncestors.assert_called_with({archiveLFN: {"Ancestors": archiveFiles.lfns}})


def test_registerDescendent_PartialFailure(archiveFiles, listOfLFNs):
    archiveFiles.lfns = listOfLFNs
    archiveLFN = "/vo/tars/myTar.tar"
    archiveFiles.parameterDict = {"RegisterDescendent": True, "ArchiveLFN": archiveLFN}
    archiveFiles.fc.addFileAncestors = Mock(
        name="AFA", side_effect=(S_ERROR("Failure"), S_OK({"Failed": {}, "Successful": {archiveLFN: "Done"}}))
    )
    archiveFiles._registerDescendent()
    archiveFiles.fc.addFileAncestors.assert_called_with({archiveLFN: {"Ancestors": archiveFiles.lfns}})


def test_registerDescendent_completeFailure(archiveFiles, listOfLFNs):
    archiveFiles.lfns = listOfLFNs
    archiveLFN = "/vo/tars/myTar.tar"
    archiveFiles.parameterDict = {"RegisterDescendent": True, "ArchiveLFN": archiveLFN}
    archiveFiles.fc.addFileAncestors = Mock(name="AFA", return_value=S_ERROR("Failure"))
    with pytest.raises(RuntimeError, match="Failed to register ancestors"):
        archiveFiles._registerDescendent()
    archiveFiles.fc.addFileAncestors.assert_called_with({archiveLFN: {"Ancestors": archiveFiles.lfns}})



#!/bin/env python
"""
Show ReqDB summary
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

__RCSID__ = "$Id$"

from DIRAC.Core.Utilities.DIRACScript import DIRACScript as Script


@Script()
def main():
    Script.parseCommandLine()

    import DIRAC

    from DIRAC.RequestManagementSystem.Client.ReqClient import ReqClient

    reqClient = ReqClient()

    dbSummary = reqClient.getDBSummary()
    if not dbSummary["OK"]:
        DIRAC.gLogger.error(dbSummary["Message"])
        DIRAC.exit(-1)

    dbSummary = dbSummary["Value"]
    if not dbSummary:
        DIRAC.gLogger.info("ReqDB is empty!")
        DIRAC.exit(0)

    reqs = dbSummary.get("Request", {})
    ops = dbSummary.get("Operation", {})
    fs = dbSummary.get("File", {})

    DIRAC.gLogger.always("Requests:")
    for reqState, reqCount in sorted(reqs.items()):
        DIRAC.gLogger.always("- '%s' %s" % (reqState, reqCount))
    DIRAC.gLogger.always("Operations:")
    for opType, opDict in sorted(ops.items()):
        DIRAC.gLogger.always("- '%s':" % opType)
        for opState, opCount in sorted(opDict.items()):
            DIRAC.gLogger.always("  - '%s' %s" % (opState, opCount))
    DIRAC.gLogger.always("Files:")
    for fState, fCount in sorted(fs.items()):
        DIRAC.gLogger.always("- '%s' %s" % (fState, fCount))

    DIRAC.exit(0)


if __name__ == "__main__":
    main()



""" Utility for prompting users
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

from DIRAC import S_OK, S_ERROR
from six.moves import input


def promptUser(message, choices=[], default="n", logger=None):
    """Prompting users with message, choices by default are 'y', 'n'"""
    if logger is None:
        from DIRAC import gLogger

        logger = gLogger

    if not choices:
        choices = ["y", "n"]
    if (choices) and (default) and (default not in choices):
        return S_ERROR("The default value is not a valid choice")
    choiceString = ""
    if choices and default:
        choiceString = "/".join(choices).replace(default, "[%s]" % default)
    elif choices and (not default):
        choiceString = "/".join(choices)
    elif (not choices) and (default):
        choiceString = "[%s]" % default

    while True:
        if choiceString:
            logger.notice("%s %s :" % (message, choiceString))
        elif default:
            logger.notice("%s %s :" % (message, default))
        else:
            logger.notice("%s :" % message)
        response = input("")
        if (not response) and (default):
            return S_OK(default)
        elif (not response) and (not default):
            logger.error("Failed to determine user selection")
            return S_ERROR("Failed to determine user selection")
        elif (response) and (choices) and (response not in choices):
            logger.notice("your answer is not valid")
            continue
        else:
            return S_OK(response)



""" Test class for JobWrapper
"""

# pylint: disable=protected-access, invalid-name

# imports
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division

import os
import shutil
import pytest
from mock import MagicMock

from DIRAC import gLogger

from DIRAC.DataManagementSystem.Client.test.mock_DM import dm_mock
from DIRAC.Resources.Catalog.test.mock_FC import fc_mock

from DIRAC.WorkloadManagementSystem.JobWrapper.JobWrapper import JobWrapper
from DIRAC.WorkloadManagementSystem.JobWrapper.Watchdog import Watchdog
from DIRAC.WorkloadManagementSystem.Client import JobStatus, JobMinorStatus

getSystemSectionMock = MagicMock()
getSystemSectionMock.return_value = "aValue"

gLogger.setLevel("DEBUG")


def test_InputData(mocker):
    mocker.patch(
        "DIRAC.WorkloadManagementSystem.JobWrapper.JobWrapper.getSystemSection", side_effect=getSystemSectionMock
    )
    mocker.patch("DIRAC.WorkloadManagementSystem.JobWrapper.JobWrapper.ModuleFactory", side_effect=MagicMock())

    jw = JobWrapper()
    jw.jobArgs["InputData"] = ""
    res = jw.resolveInputData()
    assert res["OK"] is False

    jw = JobWrapper()
    jw.jobArgs["InputData"] = "pippo"
    jw.dm = dm_mock
    jw.fc = fc_mock
    res = jw.resolveInputData()
    assert res["OK"]

    jw = JobWrapper()
    jw.jobArgs["InputData"] = "pippo"
    jw.jobArgs["LocalSE"] = "mySE"
    jw.jobArgs["InputDataModule"] = "aa.bb"
    jw.dm = dm_mock
    jw.fc = fc_mock
    res = jw.resolveInputData()
    assert res["OK"]


def test_performChecks():
    wd = Watchdog(
        pid=os.getpid(),
        exeThread=MagicMock(),
        spObject=MagicMock(),
        jobCPUTime=1000,
        memoryLimit=1024 * 1024,
        jobArgs={"StopSigNumber": 10},
    )
    res = wd._performChecks()
    assert res["OK"]


@pytest.mark.slow
def test_execute(mocker):

    mocker.patch(
        "DIRAC.WorkloadManagementSystem.JobWrapper.JobWrapper.getSystemSection", side_effect=getSystemSectionMock
    )
    mocker.patch(
        "DIRAC.WorkloadManagementSystem.JobWrapper.Watchdog.getSystemInstance", side_effect=getSystemSectionMock
    )

    jw = JobWrapper()
    jw.jobArgs = {"Executable": "/bin/ls"}
    res = jw.execute()
    print("jw.execute() returns", str(res))
    assert res["OK"]

    shutil.copy("src/DIRAC/WorkloadManagementSystem/JobWrapper/test/script-OK.sh", "script-OK.sh")
    jw = JobWrapper()
    jw.jobArgs = {"Executable": "script-OK.sh"}
    res = jw.execute()
    assert res["OK"]
    os.remove("script-OK.sh")

    shutil.copy("src/DIRAC/WorkloadManagementSystem/JobWrapper/test/script.sh", "script.sh")
    jw = JobWrapper()
    jw.jobArgs = {"Executable": "script.sh", "Arguments": "111"}
    res = jw.execute()
    assert res["OK"]  # In this case the application finished with errors,
    # but the JobWrapper executed successfully
    os.remove("script.sh")

    # this will reschedule
    shutil.copy("src/DIRAC/WorkloadManagementSystem/JobWrapper/test/script-RESC.sh", "script-RESC.sh")
    jw = JobWrapper()
    jw.jobArgs = {"Executable": "script-RESC.sh"}
    res = jw.execute()
    if res["OK"]:  # FIXME: This may happen depending on the shell - not the best test admittedly!
        print("We should not be here, unless the 'Execution thread status' is equal to 1")
        assert res["OK"]
    else:
        assert res["OK"] is False  # In this case the application finished with an error code
        # that the JobWrapper interpreted as "to reschedule"
        # so in this case the "execute" is considered an error
    os.remove("script-RESC.sh")
    os.remove("std.out")


@pytest.mark.parametrize(
    "failedFlag, expectedRes, finalStates",
    [
        (True, 1, [JobStatus.FAILED, ""]),
        (False, 0, [JobStatus.DONE, JobMinorStatus.EXEC_COMPLETE]),
    ],
)
def test_finalize(mocker, failedFlag, expectedRes, finalStates):
    mocker.patch(
        "DIRAC.WorkloadManagementSystem.JobWrapper.JobWrapper.getSystemSection", side_effect=getSystemSectionMock
    )
    mocker.patch("DIRAC.WorkloadManagementSystem.JobWrapper.JobWrapper.ModuleFactory", side_effect=MagicMock())

    jw = JobWrapper()
    jw.jobArgs = {"Executable": "/bin/ls"}
    jw.failedFlag = failedFlag

    res = jw.finalize()

    assert res == expectedRes
    assert jw.jobReport.jobStatusInfo[0][0] == finalStates[0]
    assert jw.jobReport.jobStatusInfo[0][1] == finalStates[1]



"""
  GOCDBSyncCommand module

  This command updates the downtime dates from the DowntimeCache table in case they changed
  after being fetched from GOCDB. In other words, it ensures that all the downtime dates in
  the database are current.

"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

__RCSID__ = "$Id$"

import errno
import xml.dom.minidom as minidom
from DIRAC import S_OK, S_ERROR, gLogger
from DIRAC.Core.LCG.GOCDBClient import GOCDBClient
from DIRAC.Core.LCG.GOCDBClient import _parseSingleElement
from DIRAC.ResourceStatusSystem.Command.Command import Command
from DIRAC.ResourceStatusSystem.Client.ResourceManagementClient import ResourceManagementClient


class GOCDBSyncCommand(Command):
    def __init__(self, args=None, clients=None):

        super(GOCDBSyncCommand, self).__init__(args, clients)

        if "GOCDBClient" in self.apis:
            self.gClient = self.apis["GOCDBClient"]
        else:
            self.gClient = GOCDBClient()

        if "ResourceManagementClient" in self.apis:
            self.rmClient = self.apis["ResourceManagementClient"]
        else:
            self.rmClient = ResourceManagementClient()

        self.seenHostnames = set()

    def doNew(self, masterParams=None):
        """
        Gets the downtime IDs and dates of a given hostname from the local database and compares the results
        with the remote database of GOCDB. If the downtime dates have been changed it updates the local database.

        :param: `masterParams` - string
        :return: S_OK / S_ERROR
        """

        if masterParams:
            hostname = masterParams
        else:
            return S_ERROR(errno.EINVAL, "masterParams is not provided")

        result = self.rmClient.selectDowntimeCache(name=hostname)
        if not result["OK"]:
            return result

        for downtimes in result["Value"]:

            localDBdict = {
                "DowntimeID": downtimes[3],
                "FORMATED_START_DATE": downtimes[6].strftime("%Y-%m-%d %H:%M"),
                "FORMATED_END_DATE": downtimes[7].strftime("%Y-%m-%d %H:%M"),
            }

            response = self.gClient.getHostnameDowntime(hostname, ongoing=True)

            if not response["OK"]:
                return response

            doc = minidom.parseString(response["Value"])
            downtimeElements = doc.getElementsByTagName("DOWNTIME")

            for dtElement in downtimeElements:
                GOCDBdict = _parseSingleElement(
                    dtElement, ["PRIMARY_KEY", "ENDPOINT", "FORMATED_START_DATE", "FORMATED_END_DATE"]
                )

                localDowntimeID = localDBdict["DowntimeID"]
                GOCDBDowntimeID = GOCDBdict["PRIMARY_KEY"] + " " + GOCDBdict["ENDPOINT"]

                if localDowntimeID == GOCDBDowntimeID:

                    if localDBdict["FORMATED_START_DATE"] != GOCDBdict["FORMATED_START_DATE"]:
                        result = self.rmClient.addOrModifyDowntimeCache(
                            downtimeID=localDBdict["DowntimeID"], startDate=GOCDBdict["FORMATED_START_DATE"]
                        )
                        gLogger.verbose("The start date of %s has been changed!" % downtimes[3])

                        if not result["OK"]:
                            return result

                    if localDBdict["FORMATED_END_DATE"] != GOCDBdict["FORMATED_END_DATE"]:
                        result = self.rmClient.addOrModifyDowntimeCache(
                            downtimeID=localDBdict["DowntimeID"], endDate=GOCDBdict["FORMATED_END_DATE"]
                        )
                        gLogger.verbose("The end date of %s has been changed!" % downtimes[3])

                        if not result["OK"]:
                            return result

        return S_OK()

    def doCache(self):
        return S_OK()

    def doMaster(self):
        """
        This method calls the doNew method for each hostname that exists
        in the DowntimeCache table of the local database.

        :return: S_OK / S_ERROR
        """

        # Query DB for all downtimes
        result = self.rmClient.selectDowntimeCache()
        if not result["OK"]:
            return result

        for data in result["Value"]:

            # If already processed don't do it again
            if data[0] in self.seenHostnames:
                continue

            # data[0] contains the hostname
            gLogger.verbose("Checking if the downtime of %s has been changed" % data[0])
            result = self.doNew(data[0])
            if not result["OK"]:
                return result

            self.seenHostnames.add(data[0])

        return S_OK()



""" Test_RSS_Policy_GGUSTicketsPolicy
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import unittest

import DIRAC.ResourceStatusSystem.Policy.GGUSTicketsPolicy as moduleTested


class GGUSTicketsPolicy_TestCase(unittest.TestCase):
    def setUp(self):
        """
        Setup
        """

        self.moduleTested = moduleTested
        self.testClass = self.moduleTested.GGUSTicketsPolicy

    def tearDown(self):
        """
        Tear down
        """

        del self.moduleTested
        del self.testClass


class GGUSTicketsPolicy_Success(GGUSTicketsPolicy_TestCase):
    def test_instantiate(self):
        """tests that we can instantiate one object of the tested class"""

        module = self.testClass()
        self.assertEqual("GGUSTicketsPolicy", module.__class__.__name__)

    def test_evaluate(self):
        """tests the method _evaluate"""

        module = self.testClass()

        res = module._evaluate({"OK": False, "Message": "Bo!"})
        self.assertTrue(res["OK"])
        self.assertEqual("Error", res["Value"]["Status"])
        self.assertEqual("Bo!", res["Value"]["Reason"])

        res = module._evaluate({"OK": True, "Value": None})
        self.assertTrue(res["OK"])
        self.assertEqual("Unknown", res["Value"]["Status"])
        self.assertEqual("No values to take a decision", res["Value"]["Reason"])

        res = module._evaluate({"OK": True, "Value": []})
        self.assertTrue(res["OK"])
        self.assertEqual("Unknown", res["Value"]["Status"])
        self.assertEqual("No values to take a decision", res["Value"]["Reason"])

        res = module._evaluate({"OK": True, "Value": [{"A": 1}]})
        self.assertTrue(res["OK"])
        self.assertEqual("Error", res["Value"]["Status"])
        self.assertEqual("Expected OpenTickets key for GGUSTickets", res["Value"]["Reason"])

        res = module._evaluate({"OK": True, "Value": [{"OpenTickets": 0}]})
        self.assertTrue(res["OK"])
        self.assertEqual("Active", res["Value"]["Status"])
        self.assertEqual("NO GGUSTickets unsolved", res["Value"]["Reason"])

        self.assertRaises(KeyError, module._evaluate, {"OK": True, "Value": [{"OpenTickets": 1}]})

        res = module._evaluate({"OK": True, "Value": [{"OpenTickets": 1, "Tickets": "1a"}]})
        self.assertTrue(res["OK"])
        self.assertEqual("Degraded", res["Value"]["Status"])
        self.assertEqual("1 GGUSTickets unsolved: 1a", res["Value"]["Reason"])


if __name__ == "__main__":
    suite = unittest.defaultTestLoader.loadTestsFromTestCase(GGUSTicketsPolicy_TestCase)
    suite.addTest(unittest.defaultTestLoader.loadTestsFromTestCase(GGUSTicketsPolicy_Success))
    testResult = unittest.TextTestRunner(verbosity=2).run(suite)



""" Test_RSS_Command_VOBOXAvailabilityCommand

"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import unittest
import mock

import DIRAC.ResourceStatusSystem.Command.VOBOXAvailabilityCommand as moduleTested
from six.moves import reload_module

__RCSID__ = "$Id$"

try:
    # Python 2: "reload" is built-in
    reload
except NameError:
    from importlib import reload  # pylint: disable=no-name-in-module


class VOBOXAvailabilityCommand_TestCase(unittest.TestCase):
    def setUp(self):
        """
        Setup
        """

        # Mock external libraries / modules not interesting for the unit test
        mock_RPC = mock.Mock()
        mock_RPC.ping.return_value = {"OK": True, "Value": {}}

        mock_RPCClient = mock.Mock()
        mock_RPCClient.return_value = mock_RPC
        self.mock_RPCClient = mock_RPCClient

        # Add mocks to moduleTested
        moduleTested.RPCClient = self.mock_RPCClient

        self.moduleTested = moduleTested
        self.testClass = self.moduleTested.VOBOXAvailabilityCommand

    def tearDown(self):
        """
        TearDown
        """
        del self.testClass
        del self.moduleTested
        del self.mock_RPCClient


################################################################################
# Tests


class VOBOXAvailabilityCommand_Success(VOBOXAvailabilityCommand_TestCase):
    def test_instantiate(self):
        """tests that we can instantiate one object of the tested class"""

        command = self.testClass()
        self.assertEqual("VOBOXAvailabilityCommand", command.__class__.__name__)

    def test_init(self):
        """tests that the init method does what it should do"""

        command = self.testClass()
        self.assertEqual({"onlyCache": False}, command.args)
        self.assertEqual({}, command.apis)

    def test_doCommand(self):
        """tests the doCommand method"""

        command = self.testClass()
        res = command.doCommand()

        self.assertEqual(False, res["OK"])

        command = self.testClass(args={"serviceURL": ""})
        res = command.doCommand()
        self.assertEqual(False, res["OK"])

        command = self.testClass(args={"serviceURL": "protocol://site:port/path1/path2"})
        res = command.doCommand()
        self.assertTrue(res["OK"])
        self.assertEqual(0, res["Value"]["serviceUpTime"])
        self.assertEqual(0, res["Value"]["machineUpTime"])
        self.assertEqual("site", res["Value"]["site"])
        self.assertEqual("path1", res["Value"]["system"])
        self.assertEqual("path2", res["Value"]["service"])

        mock_RPC = mock.Mock()
        mock_RPC.ping.return_value = {"OK": True, "Value": {"service uptime": 1, "host uptime": 2}}

        self.moduleTested.RPCClient.return_value = mock_RPC
        command = self.testClass(args={"serviceURL": "protocol://site:port/path1/path2"})
        res = command.doCommand()
        self.assertTrue(res["OK"])
        self.assertEqual(1, res["Value"]["serviceUpTime"])
        self.assertEqual(2, res["Value"]["machineUpTime"])
        self.assertEqual("site", res["Value"]["site"])
        self.assertEqual("path1", res["Value"]["system"])
        self.assertEqual("path2", res["Value"]["service"])

        # Restore the module
        self.moduleTested.RPCClient.return_value = self.mock_RPCClient
        reload_module(self.moduleTested)


################################################################################
# EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF



""" simple hello world job
"""

from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from DIRAC.Interfaces.API.Job import Job
from DIRAC.Interfaces.API.Dirac import Dirac
from DIRAC.DataManagementSystem.Utilities.DMSHelpers import DMSHelpers

j = Job()

j.setName("helloWorld-test")

j.setExecutable("exe-script.py", "", "Executable.log")

# <-- user settings
j.setCPUTime(172800)
tier1s = DMSHelpers().getTiers(tier=(0, 1))
j.setBannedSites(tier1s)
# user settings -->


# print j.workflow

# submit the job to dirac
result = Dirac().submitJob(j)
print(result)



#!/usr/bin/env python
########################################################################
# File :   dirac-wms-job-get-output
# Author : Stuart Paterson
########################################################################
"""
Retrieve output sandbox for a DIRAC job

Example:
  $ dirac-wms-job-get-output 1
  Job output sandbox retrieved in 1/
"""
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division

__RCSID__ = "$Id$"

import os
import shutil

import DIRAC
from DIRAC.Core.Utilities.DIRACScript import DIRACScript as Script


@Script()
def main():
    Script.registerSwitch("D:", "Dir=", "Store the output in this directory")
    Script.registerSwitch("f:", "File=", "Get output for jobs with IDs from the file")
    Script.registerSwitch("g:", "JobGroup=", "Get output for jobs in the given group")
    # Registering arguments will automatically add their description to the help menu
    Script.registerArgument(["JobID: DIRAC Job ID or a name of the file with JobID per line"], mandatory=False)
    sws, args = Script.parseCommandLine(ignoreErrors=True)

    from DIRAC.Interfaces.API.Dirac import Dirac, parseArguments
    from DIRAC.Core.Utilities.Time import toString, date, day
    from DIRAC.Core.Utilities.File import mkDir

    dirac = Dirac()
    exitCode = 0
    errorList = []

    outputDir = None
    group = None
    jobs = []
    for sw, value in sws:
        if sw in ("D", "Dir"):
            outputDir = value
        elif sw.lower() in ("f", "file"):
            if os.path.exists(value):
                jFile = open(value)
                jobs += jFile.read().split()
                jFile.close()
        elif sw.lower() in ("g", "jobgroup"):
            group = value
            jobDate = toString(date() - 30 * day)

            # Choose jobs in final state, no more than 30 days old
            result = dirac.selectJobs(jobGroup=value, date=jobDate, status="Done")
            if not result["OK"]:
                if "No jobs selected" not in result["Message"]:
                    print("Error:", result["Message"])
                    DIRAC.exit(-1)
            else:
                jobs += result["Value"]
            result = dirac.selectJobs(jobGroup=value, date=jobDate, status="Failed")
            if not result["OK"]:
                if "No jobs selected" not in result["Message"]:
                    print("Error:", result["Message"])
                    DIRAC.exit(-1)
            else:
                jobs += result["Value"]

    for arg in parseArguments(args):
        if os.path.isdir(arg):
            print("Output for job %s already retrieved, remove the output directory to redownload" % arg)
        else:
            jobs.append(arg)

    if not jobs:
        print("No jobs selected")
        DIRAC.exit(0)

    if group:
        if outputDir:
            outputDir = os.path.join(outputDir, group)
        else:
            outputDir = group

    if outputDir:
        mkDir(outputDir)
    else:
        outputDir = os.getcwd()

    jobs = [str(job) for job in jobs]
    doneJobs = os.listdir(outputDir)
    todoJobs = [job for job in jobs if job not in doneJobs]

    for job in todoJobs:

        result = dirac.getOutputSandbox(job, outputDir=outputDir)

        jobDir = str(job)
        if outputDir:
            jobDir = os.path.join(outputDir, job)
        if result["OK"]:
            if os.path.exists(jobDir):
                print("Job output sandbox retrieved in %s/" % (jobDir))
        else:
            if os.path.exists("%s" % jobDir):
                shutil.rmtree(jobDir)
            errorList.append((job, result["Message"]))
            exitCode = 2

    for error in errorList:
        print("ERROR %s: %s" % error)

    DIRAC.exit(exitCode)


if __name__ == "__main__":
    main()



""" Tornado-based HTTPs JobManager service.
"""


from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

__RCSID__ = "$Id$"

from DIRAC import gLogger
from DIRAC.Core.Tornado.Server.TornadoService import TornadoService
from DIRAC.WorkloadManagementSystem.Service.JobManagerHandler import JobManagerHandlerMixin


sLog = gLogger.getSubLogger(__name__)


class TornadoJobManagerHandler(JobManagerHandlerMixin, TornadoService):
    log = sLog

    def initializeRequest(self):
        self.diracSetup = self.get_argument("clientSetup")
        return JobManagerHandlerMixin.initializeRequest(self)



""" :mod: OperationTests
    ====================

    .. module: OperationTests
    :synopsis: Operation test cases
    .. moduleauthor:: Krzysztof.Ciba@NOSPAMgmail.com

    Operation test cases
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

__RCSID__ = "$Id $"

import pytest

from DIRAC.RequestManagementSystem.Client.File import File
from DIRAC.RequestManagementSystem.Client.Operation import Operation


def test_ctor():
    """test constructors and (de)serialisation"""
    assert isinstance(Operation(), Operation), "empty ctor failed"

    # # using fromDict
    fromDict = {
        "Type": "replicateAndRegister",
        "TargetSE": "CERN-USER,PIC-USER",
        "SourceSE": None,
    }
    operation = Operation(fromDict)
    assert isinstance(operation, Operation), "fromDict ctor failed"
    for key, value in fromDict.items():
        assert getattr(operation, key) == value, "wrong attr value %s (%s) %s" % (key, getattr(operation, key), value)

    # # same with file
    operation = Operation(fromDict)
    operation.addFile(
        File(
            {
                "LFN": "/lhcb/user/c/cibak/testFile",
                "Checksum": "1234567",
                "ChecksumType": "ADLER32",
                "Size": 1024,
                "Status": "Waiting",
            }
        )
    )

    for key, value in fromDict.items():
        assert getattr(operation, key) == value, "wrong attr value %s (%s) %s" % (key, getattr(operation, key), value)

    toJSON = operation.toJSON()
    assert toJSON["OK"], "JSON serialization failed"


def test_valid_properties():
    operation = Operation()

    operation.Arguments = "foobar"
    assert operation.Arguments == b"foobar", "wrong Arguments"

    operation.SourceSE = "CERN-RAW"
    assert operation.SourceSE == "CERN-RAW", "wrong SourceSE"

    operation.TargetSE = "CERN-RAW"
    assert operation.TargetSE == "CERN-RAW", "wrong TargetSE"

    operation.Catalog = ""
    assert operation.Catalog == "", "wrong Catalog"

    operation.Catalog = "BookkeepingDB"
    assert operation.Catalog == "BookkeepingDB", "wrong Catalog"

    operation.Error = "error"
    assert operation.Error == "error", "wrong Error"

    toJSON = operation.toJSON()
    assert toJSON["OK"]


def test_StateMachine():
    """state machine"""
    op = Operation()
    assert op.Status == "Queued", "1. wrong status %s" % op.Status

    op.addFile(File({"Status": "Waiting"}))
    assert op.Status == "Queued", "2. wrong status %s" % op.Status

    op.addFile(File({"Status": "Scheduled"}))
    assert op.Status == "Scheduled", "3. wrong status %s" % op.Status

    op.addFile(File({"Status": "Done"}))
    assert op.Status == "Scheduled", "4. wrong status %s" % op.Status

    op.addFile(File({"Status": "Failed"}))
    assert op.Status == "Scheduled", "5. wrong status %s" % op.Status

    op[3].Status = "Scheduled"
    assert op.Status == "Scheduled", "6. wrong status %s" % op.Status

    op[0].Status = "Scheduled"
    assert op.Status == "Scheduled", "7. wrong status %s" % op.Status

    op[0].Status = "Waiting"
    assert op.Status == "Scheduled", "8. wrong status %s" % op.Status

    for f in op:
        f.Status = "Done"
    assert op.Status == "Done", "9. wrong status %s" % op.Status

    for f in op:
        f.Status = "Failed"
    assert op.Status == "Failed", "9. wrong status %s" % op.Status


def test_List():
    """getitem, setitem, delitem and dirty"""
    op = Operation()

    files = []
    for _ in range(5):
        f = File()
        files.append(f)
        op += f

    for i in range(len(op)):
        assert op[i] == files[i], "__getitem__ failed"

    for i in range(len(op)):
        op[i] = File({"LFN": "/%s" % i})
        assert op[i].LFN == "/%s" % i, "__setitem__ failed"

    del op[0]
    assert len(op) == 4, "__delitem__ failed"

    # opID set
    op.OperationID = 1
    del op[0]



"""
Service handler for generating pre-signed URLs for S3 storages.
Permissions to request a URL for a given action are mapped against FC permissions
This service can serve presigned URL for any S3 storage it has the credentials for.


.. literalinclude:: ../ConfigTemplate.cfg
  :start-after: ##BEGIN S3Gateway
  :end-before: ##END
  :dedent: 2
  :caption: S3Gateway options

"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

__RCSID__ = "$Id$"

import errno

# from DIRAC
import six
from DIRAC import S_OK, S_ERROR, gLogger

from DIRAC.Core.DISET.RequestHandler import RequestHandler
from DIRAC.Core.DISET.ThreadConfig import ThreadConfig
from DIRAC.Core.Utilities.ReturnValues import returnSingleResult
from DIRAC.DataManagementSystem.Utilities.DMSHelpers import DMSHelpers
from DIRAC.Resources.Catalog.FileCatalog import FileCatalog
from DIRAC.Resources.Storage.StorageElement import StorageElement

########################################################################

LOG = gLogger.getSubLogger(__name__)


class S3GatewayHandler(RequestHandler):
    """
    .. class:: S3GatewayHandler

    """

    # FC instance to check whether access is permitted or not
    _fc = None

    # Mapping between the S3 methods and the DFC methods
    _s3ToFC_methods = {
        "head_object": "getFileMetadata",
        "get_object": "getFileMetadata",  # consider that if we are allowed to see the file metadata
        # we can also download it
        "put_object": "addFile",
        "delete_object": "removeFile",
    }

    _S3Storages = {}

    # This allows us to perform the DFC queries on behalf of a user
    # without having to recreate a DFC object every time and
    # pass it the "delegatedDN" and "delegatedGroup" values
    _tc = ThreadConfig()

    @classmethod
    def initializeHandler(cls, serviceInfoDict):
        """initialize handler"""

        log = LOG.getSubLogger("initializeHandler")

        for seName in DMSHelpers().getStorageElements():
            se = StorageElement(seName)
            # TODO: once we finally merge _allProtocolParameters with the
            # standard paramaters in the StorageBase, this will be much neater

            for storagePlugin in se.storages:
                storageParam = storagePlugin._allProtocolParameters  # pylint: disable=protected-access

                if (
                    storageParam.get("Protocol") == "s3"
                    and "Aws_access_key_id" in storageParam
                    and "Aws_secret_access_key" in storageParam
                ):

                    cls._S3Storages[seName] = storagePlugin
                    log.debug("Add %s to the list of usable S3 storages" % seName)
                    break

        log.info("S3Gateway initialized storages", "%s" % list(cls._S3Storages))

        cls._fc = FileCatalog()

        return S_OK()

    def _hasAccess(self, lfn, s3_method):
        """Check if we have permission to execute given operation on the given file (if exists) or its directory"""

        opType = self._s3ToFC_methods.get(s3_method)
        if not opType:
            return S_ERROR(errno.EINVAL, "Unknown S3 method %s" % s3_method)

        return returnSingleResult(self._fc.hasAccess(lfn, opType))

    types_createPresignedUrl = [six.string_types, six.string_types, (dict, list), six.integer_types]

    def export_createPresignedUrl(self, storageName, s3_method, urls, expiration):
        """Generate a presigned URL for a given object, given method, and given storage
        Permissions are checked against the DFC

        :param storageName: SE name
        :param s3_method: name of the S3 client method we want to perform.
        :param urls: Iterable of urls. If s3_method is put_object, it must be a dict <url:fields> where fields
                     is a dictionary (see ~DIRAC.Resources.Storage.S3Storage.S3Storage.createPresignedUrl)
        :param expiration: duration of the token
        """

        log = LOG.getSubLogger("createPresignedUrl")

        if s3_method == "put_object" and not isinstance(urls, dict):
            return S_ERROR(errno.EINVAL, "urls has to be a dict <url:fields>")

        # Fetch the remote credentials, and set them in the ThreadConfig
        # This allows to perform the FC operations on behalf of the user
        credDict = self.getRemoteCredentials()
        if not credDict:
            # If we can't obtain remote credentials, consider it permission denied
            return S_ERROR(errno.EACCES, "Could not obtain remote credentials")

        self._tc.setDN(credDict["DN"])
        self._tc.setGroup(credDict["group"])

        successful = {}
        failed = {}
        s3Plugin = self._S3Storages[storageName]
        for url in urls:
            try:
                log.verbose(
                    "Creating presigned URL",
                    "SE: %s Method: %s URL: %s Expiration: %s" % (storageName, s3_method, url, expiration),
                )

                # Finding the LFN to query the FC
                # I absolutely hate doing such path mangling but well....
                res = s3Plugin._getKeyFromURL(url)  # pylint: disable=protected-access
                if not res["OK"]:
                    failed[url] = res["Message"]
                    log.debug("Could not parse the url %s %s" % (url, res))
                    continue

                lfn = "/" + res["Value"]

                log.debug("URL: %s -> LFN %s" % (url, lfn))

                # Checking whether access is permitted
                res = self._hasAccess(lfn, s3_method)
                if not res["OK"]:
                    failed[url] = res["Message"]
                    continue

                if not res["Value"]:
                    failed[url] = "Permission denied"
                    continue

                res = returnSingleResult(
                    s3Plugin.createPresignedUrl({url: urls.get("Fields")}, s3_method, expiration=expiration)
                )

                log.debug("Presigned URL for %s: %s" % (url, res))
                if res["OK"]:
                    successful[url] = res["Value"]
                else:
                    failed["url"] = res["Message"]
            except Exception as e:
                log.exception("Exception presigning URL")
                failed[url] = repr(e)

        return S_OK({"Successful": successful, "Failed": failed})



#!/usr/bin/env python
########################################################################
# File :   dirac_admin_add_resources
# Author : Andrei Tsaregorodtsev
########################################################################
"""
Add resources from the BDII database for a given VO
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

__RCSID__ = "$Id$"

import signal
import shlex

import six

from DIRAC.Core.Utilities.DIRACScript import DIRACScript as Script
from DIRAC import gLogger, exit as DIRACExit
from DIRAC.ConfigurationSystem.Client.Utilities import getGridCEs, getSiteUpdates
from DIRAC.Core.Utilities.Subprocess import systemCall
from DIRAC.ConfigurationSystem.Client.CSAPI import CSAPI
from DIRAC.ConfigurationSystem.Client.Helpers.Path import cfgPath
from DIRAC.ConfigurationSystem.Client.Helpers.Resources import getQueues, getDIRACSiteName
from DIRAC.ConfigurationSystem.Client.Helpers.Registry import getVOOption


def processScriptSwitches():

    global vo, dry, doCEs, hostURL, onecore

    Script.registerSwitch("V:", "vo=", "Virtual Organization")
    Script.registerSwitch("D", "dry", "Dry run")
    Script.registerSwitch("C", "ce", "Process Computing Elements")
    Script.registerSwitch("H:", "host=", "use this url for information querying")
    Script.registerSwitch(
        "", "onecore", "Add Single Core Queues for each MultiCore Queue, set RequiredTag for those Queues"
    )
    Script.parseCommandLine(ignoreErrors=True)

    vo = ""
    dry = False
    doCEs = False
    hostURL = None
    onecore = False

    for sw in Script.getUnprocessedSwitches():
        if sw[0] in ("V", "vo"):
            vo = sw[1]
        if sw[0] in ("D", "dry"):
            dry = True
        if sw[0] in ("C", "ce"):
            doCEs = True
        if sw[0] in ("H", "host"):
            hostURL = sw[1]
        if sw[0] in ("onecore",):
            onecore = True


ceBdiiDict = None


def checkUnusedCEs():

    global vo, dry, ceBdiiDict, hostURL

    gLogger.notice("looking for new computing resources in the BDII database...")

    res = getQueues(community=vo)
    if not res["OK"]:
        gLogger.error("ERROR: failed to get CEs from CS", res["Message"])
        DIRACExit(-1)

    knownCEs = set()
    for _site, ces in res["Value"].items():
        knownCEs.update(ces)

    result = getGridCEs(vo, ceBlackList=knownCEs, hostURL=hostURL)
    if not result["OK"]:
        gLogger.error("ERROR: failed to get CEs from BDII", result["Message"])
        DIRACExit(-1)
    ceBdiiDict = result["BdiiInfo"]

    unknownCEs = result["UnknownCEs"]
    if unknownCEs:
        gLogger.notice("There is no (longer) information about the following CEs for the %s VO:" % vo)
        gLogger.notice("\n".join(sorted(unknownCEs)))

    siteDict = result["Value"]
    if siteDict:
        gLogger.notice("New resources available:")
        for site in siteDict:
            diracSite = "Unknown"
            result = getDIRACSiteName(site)
            if result["OK"]:
                diracSite = ",".join(result["Value"])
            if siteDict[site]:
                gLogger.notice("  %s, DIRAC site %s" % (site, diracSite))
                for ce in siteDict[site]:
                    gLogger.notice(" " * 4 + ce)
                    gLogger.notice(
                        "      %s, %s" % (siteDict[site][ce]["CEType"], "%s_%s_%s" % siteDict[site][ce]["System"])
                    )
    else:
        gLogger.notice("No new resources available, exiting")
        return

    inp = six.moves.input("\nDo you want to add sites ? [default=yes] [yes|no]: ")
    inp = inp.strip()
    if inp and inp.lower().startswith("n"):
        return

    gLogger.notice("\nAdding new sites/CEs interactively\n")

    sitesAdded = []

    for site in siteDict:
        # Get the country code:
        country = ""
        for ce in siteDict[site]:
            country = ce.strip().split(".")[-1].lower()
            if len(country) == 2:
                break
            if country == "gov":
                country = "us"
                break
        if not country or len(country) != 2:
            country = "xx"
        result = getDIRACSiteName(site)
        if not result["OK"]:
            gLogger.notice("\nThe site %s is not yet in the CS, give it a name" % site)
            diracSite = six.moves.input("[help|skip|<domain>.<name>.%s]: " % country)
            if diracSite.lower() == "skip":
                continue
            if diracSite.lower() == "help":
                gLogger.notice("%s site details:" % site)
                for k, v in ceBdiiDict[site].items():
                    if k != "CEs":
                        gLogger.notice("%s\t%s" % (k, v))
                gLogger.notice("\nEnter DIRAC site name in the form <domain>.<name>.%s\n" % country)
                diracSite = six.moves.input("[<domain>.<name>.%s]: " % country)
            try:
                _, _, _ = diracSite.split(".")
            except ValueError:
                gLogger.error("ERROR: DIRAC site name does not follow convention: %s" % diracSite)
                continue
            diracSites = [diracSite]
        else:
            diracSites = result["Value"]

        if len(diracSites) > 1:
            gLogger.notice("Attention! GOC site %s corresponds to more than one DIRAC sites:" % site)
            gLogger.notice(str(diracSites))
            gLogger.notice("Please, pay attention which DIRAC site the new CEs will join\n")

        newCEs = {}
        addedCEs = []
        for ce in siteDict[site]:
            ceType = siteDict[site][ce]["CEType"]
            for diracSite in diracSites:
                if ce in addedCEs:
                    continue
                yn = six.moves.input("Add CE %s of type %s to %s? [default yes] [yes|no]: " % (ce, ceType, diracSite))
                if yn == "" or yn.lower().startswith("y"):
                    newCEs.setdefault(diracSite, [])
                    newCEs[diracSite].append(ce)
                    addedCEs.append(ce)

        for diracSite in diracSites:
            if diracSite in newCEs:
                cmd = "dirac-admin-add-site %s %s %s" % (diracSite, site, " ".join(newCEs[diracSite]))
                gLogger.notice("\nNew site/CEs will be added with command:\n%s" % cmd)
                yn = six.moves.input("Add it ? [default yes] [yes|no]: ")
                if not (yn == "" or yn.lower().startswith("y")):
                    continue

                if dry:
                    gLogger.notice("Command is skipped in the dry run")
                else:
                    result = systemCall(0, shlex.split(cmd))
                    if not result["OK"]:
                        gLogger.error("Error while executing dirac-admin-add-site command")
                        yn = six.moves.input("Do you want to continue ? [default no] [yes|no]: ")
                        if yn == "" or yn.lower().startswith("n"):
                            if sitesAdded:
                                gLogger.notice("CEs were added at the following sites:")
                                for site, diracSite in sitesAdded:
                                    gLogger.notice("%s\t%s" % (site, diracSite))
                            DIRACExit(0)
                    else:
                        exitStatus, stdData, errData = result["Value"]
                        if exitStatus:
                            gLogger.error(
                                "Error while executing dirac-admin-add-site command\n", "\n".join([stdData, errData])
                            )
                            yn = six.moves.input("Do you want to continue ? [default no] [yes|no]: ")
                            if yn == "" or yn.lower().startswith("n"):
                                if sitesAdded:
                                    gLogger.notice("CEs were added at the following sites:")
                                    for site, diracSite in sitesAdded:
                                        gLogger.notice("%s\t%s" % (site, diracSite))
                                DIRACExit(0)
                        else:
                            sitesAdded.append((site, diracSite))
                            gLogger.notice(stdData)

    if sitesAdded:
        gLogger.notice("CEs were added at the following sites:")
        for site, diracSite in sitesAdded:
            gLogger.notice("%s\t%s" % (site, diracSite))
    else:
        gLogger.notice("No new CEs were added this time")


def updateCS(changeSet):

    global vo, dry, ceBdiiDict

    changeList = sorted(changeSet)
    if dry:
        gLogger.notice("The following needed changes are detected:\n")
    else:
        gLogger.notice("We are about to make the following changes to CS:\n")
    for entry in changeList:
        gLogger.notice("%s/%s %s -> %s" % entry)

    if not dry:
        csAPI = CSAPI()
        csAPI.initialize()
        result = csAPI.downloadCSData()
        if not result["OK"]:
            gLogger.error("Failed to initialize CSAPI object", result["Message"])
            DIRACExit(-1)
        for section, option, value, new_value in changeSet:
            if value == "Unknown" or not value:
                csAPI.setOption(cfgPath(section, option), new_value)
            else:
                csAPI.modifyValue(cfgPath(section, option), new_value)

        yn = six.moves.input("Do you want to commit changes to CS ? [default yes] [yes|no]: ")
        if yn == "" or yn.lower().startswith("y"):
            result = csAPI.commit()
            if not result["OK"]:
                gLogger.error("Error while commit to CS", result["Message"])
            else:
                gLogger.notice("Successfully committed %d changes to CS" % len(changeSet))


def updateSites():

    global vo, dry, ceBdiiDict, onecore

    result = getSiteUpdates(vo, bdiiInfo=ceBdiiDict, onecore=onecore)
    if not result["OK"]:
        gLogger.error("Failed to get site updates", result["Message"])
        DIRACExit(-1)
    changeSet = result["Value"]

    updateCS(changeSet)


def handler(signum, frame):
    gLogger.notice("\nExit is forced, bye...")
    DIRACExit(-1)


@Script()
def main():
    signal.signal(signal.SIGTERM, handler)
    signal.signal(signal.SIGINT, handler)

    global vo, dry, doCEs, ceBdiiDict

    processScriptSwitches()

    if not vo:
        gLogger.error("No VO specified")
        DIRACExit(-1)

    vo = getVOOption(vo, "VOMSName", vo)

    if doCEs:
        yn = six.moves.input("Do you want to check/add new sites to CS ? [default yes] [yes|no]: ")
        yn = yn.strip()
        if yn == "" or yn.lower().startswith("y"):
            checkUnusedCEs()

        yn = six.moves.input("Do you want to update CE details in the CS ? [default yes] [yes|no]: ")
        yn = yn.strip()
        if yn == "" or yn.lower().startswith("y"):
            updateSites()


if __name__ == "__main__":
    vo = ""
    dry = False
    doCEs = False
    ceBdiiDict = None

    main()



from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

__RCSID__ = "$Id$"

from DIRAC import gConfig
from DIRAC.ConfigurationSystem.Client.Helpers.Path import cfgPath

gBaseLocalSiteSection = "/LocalSite"


def gridEnv():
    """
    Return location of gridenv file to get a UI environment
    """
    return gConfig.getValue(cfgPath(gBaseLocalSiteSection, "GridEnv"), "")



"""
  This module contains the necessary tools to discover and load
  the handlers for serving HTTPS
"""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

__RCSID__ = "$Id$"

from tornado.web import url as TornadoURL, RequestHandler

from DIRAC import gConfig, gLogger, S_ERROR, S_OK
from DIRAC.ConfigurationSystem.Client import PathFinder
from DIRAC.Core.Base.private.ModuleLoader import ModuleLoader
from DIRAC.Core.Utilities.ObjectLoader import ObjectLoader


def urlFinder(module):
    """
    Tries to guess the url from module name.
    The URL would be of the form ``/System/Component`` (e.g. ``DataManagement/FileCatalog``)
    We search something which looks like ``<...>.<component>System.<...>.<service>Handler``

    :param module: full module name (e.g. "DIRAC.something.something")

    :returns: the deduced URL or None
    """
    sections = module.split(".")
    for section in sections:
        # This condition is a bit long
        # We search something which look like <...>.<component>System.<...>.<service>Handler
        # If find we return /<component>/<service>
        if section.endswith("System") and sections[-1].endswith("Handler"):
            return "/".join(["", section[: -len("System")], sections[-1][: -len("Handler")]])


class HandlerManager(object):
    """
    This utility class allows to load the handlers, generate the appropriate route,
    and discover the handlers based on the CS.
    In order for a service to be considered as using HTTPS, it must have
    ``protocol = https`` as an option.

    Each of the Handler will have one associated route to it:

    * Directly specified as ``LOCATION`` in the handler module
    * automatically deduced from the module name, of the form
      ``System/Component`` (e.g. ``DataManagement/FileCatalog``)
    """

    def __init__(self, autoDiscovery=True):
        """
        Initialization function, you can set autoDiscovery=False to prevent automatic
        discovery of handler. If disabled you can use loadHandlersByServiceName() to
        load your handlers or loadHandlerInHandlerManager()

        :param autoDiscovery: (default True) Disable the automatic discovery,
            can be used to choose service we want to load.
        """
        self.__handlers = {}
        self.__objectLoader = ObjectLoader()
        self.__autoDiscovery = autoDiscovery
        self.loader = ModuleLoader("Service", PathFinder.getServiceSection, RequestHandler, moduleSuffix="Handler")

    def __addHandler(self, handlerTuple, url=None):
        """
        Function which add handler to list of known handlers


        :param handlerTuple: (path, class)
        """
        # Check if handler not already loaded
        if not url or url not in self.__handlers:
            gLogger.debug("Find new handler %s" % (handlerTuple[0]))

            # If url is not given, try to discover it
            if url is None:
                # FIRST TRY: Url is hardcoded
                try:
                    url = handlerTuple[1].LOCATION
                # SECOND TRY: URL can be deduced from path
                except AttributeError:
                    gLogger.debug("No location defined for %s try to get it from path" % handlerTuple[0])
                    url = urlFinder(handlerTuple[0])

            # We add "/" if missing at begin, e.g. we found "Framework/Service"
            # URL can't be relative in Tornado
            if url and not url.startswith("/"):
                url = "/%s" % url
            elif not url:
                gLogger.warn("URL not found for %s" % (handlerTuple[0]))
                return S_ERROR("URL not found for %s" % (handlerTuple[0]))

            # Finally add the URL to handlers
            if url not in self.__handlers:
                self.__handlers[url] = handlerTuple[1]
                gLogger.info("New handler: %s with URL %s" % (handlerTuple[0], url))
        else:
            gLogger.debug("Handler already loaded %s" % (handlerTuple[0]))
        return S_OK()

    def discoverHandlers(self):
        """
        Force the discovery of URL, automatic call when we try to get handlers for the first time.
        You can disable the automatic call with autoDiscovery=False at initialization
        """
        gLogger.debug("Trying to auto-discover the handlers for Tornado")

        # Look in config
        diracSystems = gConfig.getSections("/Systems")
        serviceList = []
        if diracSystems["OK"]:
            for system in diracSystems["Value"]:
                try:
                    instance = PathFinder.getSystemInstance(system)
                    services = gConfig.getSections("/Systems/%s/%s/Services" % (system, instance))
                    if services["OK"]:
                        for service in services["Value"]:
                            newservice = "%s/%s" % (system, service)

                            # We search in the CS all handlers which used HTTPS as protocol
                            isHTTPS = gConfig.getValue(
                                "/Systems/%s/%s/Services/%s/Protocol" % (system, instance, service)
                            )
                            if isHTTPS and isHTTPS.lower() == "https":
                                serviceList.append(newservice)
                # On systems sometime you have things not related to services...
                except RuntimeError:
                    pass
        return self.loadHandlersByServiceName(serviceList)

    def loadHandlersByServiceName(self, servicesNames):
        """
        Load a list of handler from list of service using DIRAC moduleLoader
        Use :py:class:`DIRAC.Core.Base.private.ModuleLoader`

        :param servicesNames: list of service, e.g. ['Framework/Hello', 'Configuration/Server']
        """

        # Use DIRAC system to load: search in CS if path is given and if not defined
        # it search in place it should be (e.g. in DIRAC/FrameworkSystem/Service)
        if not isinstance(servicesNames, list):
            servicesNames = [servicesNames]

        load = self.loader.loadModules(servicesNames)
        if not load["OK"]:
            return load
        for module in self.loader.getModules().values():
            url = module["loadName"]

            # URL can be like https://domain:port/service/name or just service/name
            # Here we just want the service name, for tornado
            serviceTuple = url.replace("https://", "").split("/")[-2:]
            url = "%s/%s" % (serviceTuple[0], serviceTuple[1])
            self.__addHandler((module["loadName"], module["classObj"]), url)
        return S_OK()

    def getHandlersURLs(self):
        """
        Get all handler for usage in Tornado, as a list of tornado.web.url
        If there is no handler found before, it try to find them

        :returns: a list of URL (not the string with "https://..." but the tornado object)
                  see http://www.tornadoweb.org/en/stable/web.html#tornado.web.URLSpec
        """
        if not self.__handlers and self.__autoDiscovery:
            self.__autoDiscovery = False
            self.discoverHandlers()
        urls = []
        for key in self.__handlers:
            urls.append(TornadoURL(key, self.__handlers[key]))
        return urls

    def getHandlersDict(self):
        """
        Return all handler dictionary

        :returns: dictionary with absolute url as key ("/System/Service")
                  and tornado.web.url object as value
        """
        if not self.__handlers and self.__autoDiscovery:
            self.__autoDiscovery = False
            res = self.discoverHandlers()
            if not res["OK"]:
                gLogger.error("Could not load handlers", res)
        return self.__handlers



"""
LogLevels wrapper
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

__RCSID__ = "$Id$"

import logging


class LogLevels(object):
    """
    Wrapper of the old LogLevels class.
    LogLevels is used to integrate custom levels to logging: verbose, notice and always.

    It is useful to make conversion string-integer.
    In fact, logging use only integers while the oldgLogger used strings, so we need a converter.
    Example: log.setLevel(logging.ALWAYS) in 'logging' become log.setLevel("always") in gLogger.
    We keep the string form because there are many and many calls with string levels.

    There is a method named getLevelName in 'logging' which could do almost the same job, but with logging,
    we can not return all level names in one time
    and the method getLevelName does not return errors or None values when a level
    does not exist. So at the moment, the LogLevels class is useful.
    """

    DEBUG = logging.DEBUG
    VERBOSE = 15
    INFO = logging.INFO
    WARN = logging.WARNING
    NOTICE = 35
    ERROR = logging.ERROR
    ALWAYS = 45
    FATAL = logging.CRITICAL

    __levelDict = {
        "DEBUG": DEBUG,
        "VERBOSE": VERBOSE,
        "INFO": INFO,
        "WARN": WARN,
        "NOTICE": NOTICE,
        "ERROR": ERROR,
        "ALWAYS": ALWAYS,
        "FATAL": FATAL,
    }

    @classmethod
    def getLevelValue(cls, sName):
        """
        Get a level value from a level name.
        We could use logging.getLevelName() to get the level value but it is less simple.

        :param str sName: level name
        :return: a level value according to a level name
        """
        return cls.__levelDict.get(sName.upper())

    @classmethod
    def getLevel(cls, level):
        """
        Get a level name from a level value.
        We could use logging.getLevelName() to get the level value but it is less simple.

        :param int level: level value
        :return: a level name according to a level value
        """
        for lev in cls.__levelDict:
            if cls.__levelDict[lev] == level:
                return lev
        return None

    @classmethod
    def getLevelNames(cls):
        """
        :return: all level names available in the wrapper
        """
        return list(cls.__levelDict)

    @classmethod
    def getLevels(cls):
        """
        :return: the level dictionary. Must no be redefined
        """
        return cls.__levelDict



#!/usr/bin/env python
########################################################################
# File :    dirac-admin-service-ports
# Author :  Stuart Paterson
########################################################################
"""
Print the service ports for the specified setup

Example:
  $ dirac-admin-service-ports
  {'Framework/ProxyManager': 9152,
   'Framework/SystemAdministrator': 9162,
   'Framework/UserProfileManager': 9155,
   'WorkloadManagement/JobManager': 9132,
   'WorkloadManagement/PilotManager': 9171,
   'WorkloadManagement/Matcher': 9170,
   'WorkloadManagement/SandboxStore': 9196,
   'WorkloadManagement/WMSAdministrator': 9145}
"""
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division

__RCSID__ = "$Id$"

import DIRAC
from DIRAC.Core.Utilities.DIRACScript import DIRACScript as Script


@Script()
def main():
    # Registering arguments will automatically add their description to the help menu
    Script.registerArgument("Setup:    Name of the setup", default="", mandatory=False)
    Script.parseCommandLine(ignoreErrors=True)
    setup = Script.getPositionalArgs(group=True)

    from DIRAC.Interfaces.API.DiracAdmin import DiracAdmin

    diracAdmin = DiracAdmin()
    result = diracAdmin.getServicePorts(setup, printOutput=True)
    if result["OK"]:
        DIRAC.exit(0)
    else:
        print(result["Message"])
        DIRAC.exit(2)


if __name__ == "__main__":
    main()



########################################################################
# $HeadURL $
# File: PutAndRegister.py
# Author: Krzysztof.Ciba@NOSPAMgmail.com
# Date: 2013/03/25 07:43:24
########################################################################

""" :mod: PutAndRegister

    ====================

    .. module: PutAndRegister

    :synopsis: putAndRegister operation handler

    .. moduleauthor:: Krzysztof.Ciba@NOSPAMgmail.com

    PutAndRegister operation handler
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

__RCSID__ = "$Id $"

# #
# @file PutAndRegister.py
# @author Krzysztof.Ciba@NOSPAMgmail.com
# @date 2013/03/25 07:43:34
# @brief Definition of PutAndRegister class.

# # imports
from DIRAC import S_OK, S_ERROR
from DIRAC.FrameworkSystem.Client.MonitoringClient import gMonitor
from DIRAC.DataManagementSystem.Agent.RequestOperations.DMSRequestOperationsBase import DMSRequestOperationsBase
from DIRAC.DataManagementSystem.Client.DataManager import DataManager

from DIRAC.MonitoringSystem.Client.MonitoringReporter import MonitoringReporter

########################################################################


class PutAndRegister(DMSRequestOperationsBase):
    """
    .. class:: PutAndRegister

    PutAndRegister operation handler.

    This takes a local file and put it on a StorageElement before registering it in a catalog

    """

    def __init__(self, operation=None, csPath=None):
        """c'tor

        :param self: self reference
        :param Operation operation: Operation instance
        :param str csPath: CS path for this handler
        """
        # # base classes ctor
        super(PutAndRegister, self).__init__(operation, csPath)

        self.dm = DataManager()

    def __call__(self):
        """PutAndRegister operation processing"""

        # The flag  'rmsMonitoring' is set by the RequestTask and is False by default.
        # Here we use 'createRMSRecord' to create the ES record which is defined inside OperationHandlerBase.
        if self.rmsMonitoring:
            self.rmsMonitoringReporter = MonitoringReporter(monitoringType="RMSMonitoring")
        else:
            # # gMonitor stuff
            gMonitor.registerActivity(
                "PutAtt", "File put attempts", "RequestExecutingAgent", "Files/min", gMonitor.OP_SUM
            )
            gMonitor.registerActivity(
                "PutFail", "Failed file puts", "RequestExecutingAgent", "Files/min", gMonitor.OP_SUM
            )
            gMonitor.registerActivity(
                "PutOK", "Successful file puts", "RequestExecutingAgent", "Files/min", gMonitor.OP_SUM
            )
            gMonitor.registerActivity(
                "RegisterOK", "Successful file registrations", "RequestExecutingAgent", "Files/min", gMonitor.OP_SUM
            )
            gMonitor.registerActivity(
                "RegisterFail", "Failed file registrations", "RequestExecutingAgent", "Files/min", gMonitor.OP_SUM
            )

        # # list of targetSEs
        targetSEs = self.operation.targetSEList

        if len(targetSEs) != 1:
            self.log.error("Wrong value for TargetSE list, should contain only one target!", "%s" % targetSEs)
            self.operation.Error = "Wrong parameters: TargetSE should contain only one targetSE"
            for opFile in self.operation:

                opFile.Status = "Failed"
                opFile.Error = "Wrong parameters: TargetSE should contain only one targetSE"

                if not self.rmsMonitoring:
                    gMonitor.addMark("PutAtt", 1)
                    gMonitor.addMark("PutFail", 1)

            if self.rmsMonitoring:
                for status in ["Attempted", "Failed"]:
                    self.rmsMonitoringReporter.addRecord(self.createRMSRecord(status, len(self.operation)))
                    self.rmsMonitoringReporter.commit()

            return S_ERROR("TargetSE should contain only one target, got %s" % targetSEs)

        targetSE = targetSEs[0]
        bannedTargets = self.checkSEsRSS(targetSE)
        if not bannedTargets["OK"]:
            if self.rmsMonitoring:
                for status in ["Attempted", "Failed"]:
                    self.rmsMonitoringReporter.addRecord(self.createRMSRecord(status, len(self.operation)))
                self.rmsMonitoringReporter.commit()
            else:
                gMonitor.addMark("PutAtt")
                gMonitor.addMark("PutFail")
            return bannedTargets

        if bannedTargets["Value"]:
            return S_OK("%s targets are banned for writing" % ",".join(bannedTargets["Value"]))

        # # get waiting files
        waitingFiles = self.getWaitingFilesList()

        if self.rmsMonitoring:
            self.rmsMonitoringReporter.addRecord(self.createRMSRecord("Attempted", len(waitingFiles)))

        # # loop over files
        for opFile in waitingFiles:
            # # get LFN
            lfn = opFile.LFN
            self.log.info("processing file %s" % lfn)

            if not self.rmsMonitoring:
                gMonitor.addMark("PutAtt", 1)

            pfn = opFile.PFN
            guid = opFile.GUID
            checksum = opFile.Checksum

            # # call DataManager passing a list of requested catalogs
            catalogs = self.operation.Catalog
            if catalogs:
                catalogs = [cat.strip() for cat in catalogs.split(",")]
            putAndRegister = DataManager(catalogs=catalogs).putAndRegister(
                lfn, pfn, targetSE, guid=guid, checksum=checksum
            )
            if not putAndRegister["OK"]:
                if self.rmsMonitoring:
                    self.rmsMonitoringReporter.addRecord(self.createRMSRecord("Failed", 1))
                else:
                    gMonitor.addMark("PutFail", 1)
                #         self.dataLoggingClient().addFileRecord( lfn, "PutFail", targetSE, "", "PutAndRegister" )
                self.log.error("Completely failed to put and register file", putAndRegister["Message"])
                opFile.Error = str(putAndRegister["Message"])
                self.operation.Error = str(putAndRegister["Message"])
                continue

            putAndRegister = putAndRegister["Value"]

            if lfn in putAndRegister["Failed"]:
                if self.rmsMonitoring:
                    self.rmsMonitoringReporter.addRecord(self.createRMSRecord("Failed", 1))
                else:
                    gMonitor.addMark("PutFail", 1)
                #         self.dataLoggingClient().addFileRecord( lfn, "PutFail", targetSE, "", "PutAndRegister" )

                reason = putAndRegister["Failed"][lfn]
                self.log.error("Failed to put and register file", " %s at %s: %s" % (lfn, targetSE, reason))
                opFile.Error = str(reason)
                self.operation.Error = str(reason)
                continue

            putAndRegister = putAndRegister["Successful"]
            if lfn in putAndRegister:

                if "put" not in putAndRegister[lfn]:

                    if self.rmsMonitoring:
                        self.rmsMonitoringReporter.addRecord(self.createRMSRecord("Failed", 1))
                    else:
                        gMonitor.addMark("PutFail", 1)
                    #           self.dataLoggingClient().addFileRecord( lfn, "PutFail", targetSE, "", "PutAndRegister" )

                    self.log.info("failed to put %s to %s" % (lfn, targetSE))

                    opFile.Error = "put failed"
                    self.operation.Error = "put failed"
                    continue

                if "register" not in putAndRegister[lfn]:

                    if self.rmsMonitoring:
                        self.rmsMonitoringReporter.addRecord(self.createRMSRecord("Failed", 1))
                    else:
                        gMonitor.addMark("PutOK", 1)
                        gMonitor.addMark("RegisterFail", 1)

                    #           self.dataLoggingClient().addFileRecord( lfn, "Put", targetSE, "", "PutAndRegister" )
                    #           self.dataLoggingClient().addFileRecord( lfn, "RegisterFail", targetSE, "", "PutAndRegister" )

                    self.log.info("put of %s to %s took %s seconds" % (lfn, targetSE, putAndRegister[lfn]["put"]))
                    self.log.error("Register of lfn to SE failed", "%s to %s" % (lfn, targetSE))

                    opFile.Error = "failed to register %s at %s" % (lfn, targetSE)
                    opFile.Status = "Failed"

                    self.log.info(opFile.Error)
                    registerOperation = self.getRegisterOperation(opFile, targetSE)
                    self.request.insertAfter(registerOperation, self.operation)
                    continue

                if self.rmsMonitoring:
                    self.rmsMonitoringReporter.addRecord(self.createRMSRecord("Successful", 1))
                else:
                    gMonitor.addMark("PutOK", 1)
                    gMonitor.addMark("RegisterOK", 1)

                #         self.dataLoggingClient().addFileRecord( lfn, "Put", targetSE, "", "PutAndRegister" )
                #         self.dataLoggingClient().addFileRecord( lfn, "Register", targetSE, "", "PutAndRegister" )

                opFile.Status = "Done"
                for op in ("put", "register"):
                    self.log.info("%s of %s to %s took %s seconds" % (op, lfn, targetSE, putAndRegister[lfn][op]))

        if self.rmsMonitoring:
            self.rmsMonitoringReporter.commit()

        return S_OK()



#!/usr/bin/env python
"""
Add or Modify a Host info in DIRAC

Example:
  $ dirac-admin-add-host -H dirac.i2np3.fr -D /O=GRID-FR/C=FR/O=CNRS/OU=CC-IN2P3/CN=dirac.in2p3.fr
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

__RCSID__ = "$Id$"

import DIRAC
from DIRAC import gLogger
from DIRAC.Core.Utilities.DIRACScript import DIRACScript as Script

hostName = None
hostDN = None
hostProperties = []


def setHostName(arg):
    global hostName
    if hostName or not arg:
        Script.showHelp(exitCode=1)
    hostName = arg


def setHostDN(arg):
    global hostDN
    if hostDN or not arg:
        Script.showHelp(exitCode=1)
    hostDN = arg


def addProperty(arg):
    global hostProperties
    if not arg:
        Script.showHelp(exitCode=1)
    if arg not in hostProperties:
        hostProperties.append(arg)


@Script()
def main():
    global hostName
    global hostDN
    global hostProperties
    Script.registerSwitch("H:", "HostName:", "Name of the Host (Mandatory)", setHostName)
    Script.registerSwitch("D:", "HostDN:", "DN of the Host Certificate (Mandatory)", setHostDN)
    Script.registerSwitch(
        "P:", "Property:", "Property to be added to the Host (Allow Multiple instances or None)", addProperty
    )
    # Registering arguments will automatically add their description to the help menu
    Script.registerArgument(
        ["Property=<Value>: Other properties to be added to the Host like (Responsible=XXX)"], mandatory=False
    )

    _, args = Script.parseCommandLine(ignoreErrors=True)

    if hostName is None or hostDN is None:
        Script.showHelp(exitCode=1)

    from DIRAC.Interfaces.API.DiracAdmin import DiracAdmin

    diracAdmin = DiracAdmin()
    exitCode = 0
    errorList = []

    hostProps = {"DN": hostDN}
    if hostProperties:
        hostProps["Properties"] = ", ".join(hostProperties)

    for prop in args:
        pl = prop.split("=")
        if len(pl) < 2:
            errorList.append(("in arguments", "Property %s has to include a '=' to separate name from value" % prop))
            exitCode = 255
        else:
            pName = pl[0]
            pValue = "=".join(pl[1:])
            gLogger.info("Setting property %s to %s" % (pName, pValue))
            hostProps[pName] = pValue

    if not diracAdmin.csModifyHost(hostName, hostProps, createIfNonExistant=True)["OK"]:
        errorList.append(("add host", "Cannot register host %s" % hostName))
        exitCode = 255
    else:
        result = diracAdmin.csCommitChanges()
        if not result["OK"]:
            errorList.append(("commit", result["Message"]))
            exitCode = 255

    if exitCode == 0:
        from DIRAC.FrameworkSystem.Client.ComponentMonitoringClient import ComponentMonitoringClient

        cmc = ComponentMonitoringClient()
        ret = cmc.hostExists(dict(HostName=hostName))
        if not ret["OK"]:
            gLogger.error("Cannot check if host is registered in ComponentMonitoring", ret["Message"])
        elif ret["Value"]:
            gLogger.info("Host already registered in ComponentMonitoring")
        else:
            ret = cmc.addHost(dict(HostName=hostName, CPU="TO_COME"))
            if not ret["OK"]:
                gLogger.error("Failed to add Host to ComponentMonitoring", ret["Message"])

    for error in errorList:
        gLogger.error("%s: %s" % error)

    DIRAC.exit(exitCode)


if __name__ == "__main__":
    main()



""" A mock of the DataManager, used for testing purposes
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

# pylint: disable=protected-access, missing-docstring, invalid-name, line-too-long

from mock import MagicMock

from DIRAC import S_OK

dm_mock = MagicMock()
dm_mock.getReplicas.return_value = S_OK(
    {
        "Successful": {
            "/a/lfn/1.txt": {"SE1": "/a/lfn/at/SE1.1.txt", "SE2": "/a/lfn/at/SE2.1.txt"},
            "/a/lfn/2.txt": {"SE1": "/a/lfn/at/SE1.1.txt"},
        },
        "Failed": {},
    }
)
dm_mock.getActiveReplicas.return_value = dm_mock.getReplicas.return_value
dm_mock.getReplicasForJobs.return_value = dm_mock.getReplicas.return_value
dm_mock.getCatalogFileMetadata.return_value = {
    "OK": True,
    "Value": {"Successful": {"pippo": "metadataPippo"}, "Failed": None},
}
dm_mock.removeFile.return_value = {"OK": True, "Value": {"Failed": False}}
dm_mock.putStorageDirectory.return_value = {"OK": True, "Value": {"Failed": False}}
dm_mock.addCatalogFile.return_value = {"OK": True, "Value": {"Failed": False}}
dm_mock.putAndRegister.return_value = {"OK": True, "Value": {"Failed": False}}
dm_mock.getFile.return_value = {"OK": True, "Value": {"Failed": False}}



""" TransferCommand module
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

__RCSID__ = "$Id$"


from datetime import datetime, timedelta

from DIRAC import S_OK, S_ERROR
from DIRAC.Core.Utilities.JEncode import strToIntDict
from DIRAC.ConfigurationSystem.Client.Helpers.Resources import getSites
from DIRAC.AccountingSystem.Client.ReportsClient import ReportsClient
from DIRAC.DataManagementSystem.Utilities.DMSHelpers import DMSHelpers
from DIRAC.ResourceStatusSystem.Client.ResourceManagementClient import ResourceManagementClient
from DIRAC.ResourceStatusSystem.Command.Command import Command


class TransferCommand(Command):
    """
    Transfer "master" Command
    """

    def __init__(self, args=None, clients=None):

        super(TransferCommand, self).__init__(args, clients)

        if "ReportsClient" in self.apis:
            self.rClient = self.apis["ReportsClient"]
        else:
            self.rClient = ReportsClient()

        if "ResourceManagementClient" in self.apis:
            self.rmClient = self.apis["ResourceManagementClient"]
        else:
            self.rmClient = ResourceManagementClient()

    def _storeCommand(self, results):
        """
        Stores the results of doNew method on the database.
        """

        for result in results:

            resQuery = self.rmClient.addOrModifyTransferCache(
                result["SourceName"], result["DestinationName"], result["Metric"], result["Value"]
            )
            if not resQuery["OK"]:
                return resQuery
        return S_OK()

    def _prepareCommand(self):
        """
        TransferChannelCommand requires four arguments:
        - hours       : <int>
        - direction   : Source | Destination
        - elementName : <str>
        - metric      : Quality | FailedTransfers

        GGUSTickets are associated with gocDB names, so we have to transform the
        diracSiteName into a gocSiteName.
        """

        if "hours" not in self.args:
            return S_ERROR("Number of hours not specified")
        hours = self.args["hours"]

        if "direction" not in self.args:
            return S_ERROR("direction is missing")
        direction = self.args["direction"]

        if direction not in ["Source", "Destination"]:
            return S_ERROR("direction is not Source nor Destination")

        if "name" not in self.args:
            return S_ERROR('"name" is missing')
        name = self.args["name"]

        if "metric" not in self.args:
            return S_ERROR("metric is missing")
        metric = self.args["metric"]

        if metric not in ["Quality", "FailedTransfers"]:
            return S_ERROR("metric is not Quality nor FailedTransfers")

        return S_OK((hours, name, direction, metric))

    def doNew(self, masterParams=None):
        """
        Gets the parameters to run, either from the master method or from its
        own arguments.

        For every elementName ( cannot process bulk queries.. ) contacts the
        accounting client. It reurns dictionaries like { 'X -> Y' : { id: 100%.. } }

        If there are ggus tickets, are recorded and then returned.
        """

        if masterParams is not None:
            hours, name, direction, metric = masterParams

        else:
            params = self._prepareCommand()
            if not params["OK"]:
                return params
            hours, name, direction, metric = params["Value"]

        toD = datetime.utcnow()
        fromD = toD - timedelta(hours=hours)

        # dictionary with conditions for the accounting
        transferDict = {"OperationType": "putAndRegister", direction: name}

        if metric == "FailedTransfers":
            transferDict["FinalStatus"] = ["Failed"]

        transferResults = self.rClient.getReport("DataOperation", metric, fromD, toD, transferDict, "Channel")

        if not transferResults["OK"]:
            return transferResults
        transferResults = transferResults["Value"]

        if "data" not in transferResults:
            return S_ERROR("Missing data key")

        transferResults = {channel: strToIntDict(value) for channel, value in transferResults["data"].items()}

        uniformResult = []

        for channel, elementDict in transferResults.items():

            try:
                source, destination = channel.split(" -> ")
            except ValueError:
                continue

            channelDict = {}
            channelDict["SourceName"] = source
            channelDict["DestinationName"] = destination
            channelDict["Metric"] = metric
            channelDict["Value"] = int(sum(elementDict.values()) / len(elementDict.values()))

            uniformResult.append(channelDict)

        storeRes = self._storeCommand(uniformResult)
        if not storeRes["OK"]:
            return storeRes

        # Compute mean of all transfer channels
        value = 0
        for channelDict in uniformResult:
            value += channelDict["Value"]

        if uniformResult:
            value = float(value) / len(uniformResult)
        else:
            value = None

        return S_OK({"Mean": value, "Name": name})

    def doCache(self):
        """
        Method that reads the cache table and tries to read from it. It will
        return a list of dictionaries if there are results.
        """

        params = self._prepareCommand()
        if not params["OK"]:
            return params
        _hours, name, direction, metric = params["Value"]

        sourceName, destinationName = None, None
        if direction == "Source":
            sourceName = name
        if direction == "Destination":
            destinationName = name

        result = self.rmClient.selectTransferCache(sourceName, destinationName, metric)
        if not result["OK"]:
            return result

        result = [dict(zip(result["Columns"], res)) for res in result["Value"]]

        # Compute mean of all transfer channels
        value = 0
        for channelDict in result:
            value += channelDict["Value"]

        if result:
            value = float(value) / len(result)
        else:
            value = None

        return S_OK({"Mean": value, "Name": name})

    def doMaster(self):
        """
        Master method, which looks little bit spaguetti code, sorry !
        - It gets all Sites.
        - It gets all StorageElements

        As there is no bulk query, it compares with what we have on the database.
        It queries a portion of them.
        """

        sites = getSites()
        if not sites["OK"]:
            return sites
        sites = sites["Value"]

        elementNames = sites + DMSHelpers().getStorageElements()

        #    sourceQuery = self.rmClient.selectTransferCache( meta = { 'columns' : [ 'SourceName' ] } )
        #    if not sourceQuery[ 'OK' ]:
        #      return sourceQuery
        #    sourceQuery = [ element[0] for element in sourceQuery[ 'Value' ] ]
        #
        #    sourceElementsToQuery = list( set( elementNames ).difference( set( sourceQuery ) ) )
        self.log.info("Processing %s" % ", ".join(elementNames))

        for metric in ["Quality", "FailedTransfers"]:
            for direction in ["Source", "Destination"]:
                # 2 hours of window
                result = self.doNew((2, elementNames, direction, metric))
                if not result["OK"]:
                    self.metrics["failed"].append(result)

        return S_OK(self.metrics)



""" DIRAC FileCatalog Storage Element Manager mix-in class for SE definitions within the FC database"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

__RCSID__ = "$Id$"

import six
import time
import random

from DIRAC import S_OK, S_ERROR, gConfig, gLogger
from DIRAC.Core.Utilities.Pfn import pfnunparse
from DIRAC.DataManagementSystem.DB.FileCatalogComponents.SEManager.SEManagerBase import SEManagerBase


class SEManagerDB(SEManagerBase):
    def _refreshSEs(self, connection=False):

        req = "SELECT SEID,SEName FROM FC_StorageElements;"
        res = self.db._query(req)
        if not res["OK"]:
            return res
        seNames = set([se[1] for se in res["Value"]])

        # If there are no changes between the DB and the cache
        # and we updated the cache recently enough, just return
        if seNames == set(self.db.seNames) and ((time.time() - self.lastUpdate) < self.seUpdatePeriod):
            return S_OK()

        startTime = time.time()
        self.lock.acquire()
        waitTime = time.time()
        gLogger.debug("SEManager RefreshSEs lock created. Waited %.3f seconds." % (waitTime - startTime))

        # Check once more the lastUpdate because it could have been updated while we were waiting for the lock
        if (time.time() - self.lastUpdate) < self.seUpdatePeriod:
            self.lock.release()
            return S_OK()

        for seName, seId in list(self.db.seNames.items()):
            if seName not in seNames:
                del self.db.seNames[seName]
                del self.db.seids[seId]

        for seid, seName in res["Value"]:
            self.db.seNames[seName] = seid
            self.db.seids[seid] = seName
        gLogger.debug("SEManager RefreshSEs lock released. Used %.3f seconds." % (time.time() - waitTime))

        # Update the lastUpdate time
        self.lastUpdate = time.time()
        self.lock.release()
        return S_OK()

    def __addSE(self, seName, connection=False):
        startTime = time.time()
        self.lock.acquire()
        waitTime = time.time()
        gLogger.debug("SEManager AddSE lock created. Waited %.3f seconds. %s" % (waitTime - startTime, seName))
        if seName in self.db.seNames.keys():
            seid = self.db.seNames[seName]
            gLogger.debug("SEManager AddSE lock released. Used %.3f seconds. %s" % (time.time() - waitTime, seName))
            self.lock.release()
            return S_OK(seid)
        connection = self.db._getConnection()
        res = self.db.insertFields("FC_StorageElements", ["SEName"], [seName], connection)
        if not res["OK"]:
            gLogger.debug("SEManager AddSE lock released. Used %.3f seconds. %s" % (time.time() - waitTime, seName))
            self.lock.release()
            if "Duplicate entry" in res["Message"]:
                result = self._refreshSEs(connection)
                if not result["OK"]:
                    return result
                if seName in self.db.seNames.keys():
                    seid = self.db.seNames[seName]
                    return S_OK(seid)
            return res
        seid = res["lastRowId"]
        self.db.seids[seid] = seName
        self.db.seNames[seName] = seid
        gLogger.debug("SEManager AddSE lock released. Used %.3f seconds. %s" % (time.time() - waitTime, seName))
        self.lock.release()
        return S_OK(seid)

    def __removeSE(self, seName, connection=False):
        connection = self.db._getConnection()
        startTime = time.time()
        self.lock.acquire()
        waitTime = time.time()
        gLogger.debug("SEManager RemoveSE lock created. Waited %.3f seconds. %s" % (waitTime - startTime, seName))
        seid = self.db.seNames.get(seName, "Missing")
        req = "DELETE FROM FC_StorageElements WHERE SEName='%s'" % seName
        res = self.db._update(req, connection)
        if not res["OK"]:
            gLogger.debug("SEManager RemoveSE lock released. Used %.3f seconds. %s" % (time.time() - waitTime, seName))
            self.lock.release()
            return res
        if seid != "Missing":
            self.db.seNames.pop(seName)
            self.db.seids.pop(seid)
        gLogger.debug("SEManager RemoveSE lock released. Used %.3f seconds. %s" % (time.time() - waitTime, seName))
        self.lock.release()
        return S_OK()

    def findSE(self, seName):
        return self.getSEID(seName)

    def getSEID(self, seName):
        """Get ID for a SE specified by its name"""
        if isinstance(seName, six.integer_types):
            return S_OK(seName)
        if seName in self.db.seNames.keys():
            return S_OK(self.db.seNames[seName])
        return self.__addSE(seName)

    def addSE(self, seName):
        return self.getSEID(seName)

    def getSEName(self, seID):
        """Return the name of the SE.

        Refresh list of SEs if not found, an SE might have been added by a different FileCatalog instance.

        :param int seID: ID of a storage element
        :return: S_OK/S_ERROR
        """
        if seID in self.db.seids:
            return S_OK(self.db.seids[seID])
        gLogger.info("getSEName: seID not found, refreshing", "ID: %s" % seID)
        result = self._refreshSEs(connection=False)
        if not result["OK"]:
            gLogger.error("getSEName: refreshing failed", result["Message"])
            return result
        if seID in self.db.seids:
            return S_OK(self.db.seids[seID])
        gLogger.error("getSEName: seID not found after refreshing", "ID: %s" % seID)
        return S_ERROR("SE id %d not found" % seID)

    def deleteSE(self, seName, force=True):
        # ToDo: Check first if there are replicas using this SE
        if not force:
            pass
        return self.__removeSE(seName)



""" few utilities
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import os
import shutil

# pylint: disable=missing-docstring


def cleanTestDir():
    for fileIn in os.listdir("."):
        if "Local" in fileIn and os.path.isdir(fileIn):
            shutil.rmtree(fileIn)
    for fileToRemove in ["std.out", "std.err"]:
        try:
            os.remove(fileToRemove)
        except OSError:
            continue


def find_all(name, path, directory=None):
    """Simple files finder"""
    result = []
    for root, _dirs, files in os.walk(path):
        if name in files:
            result.append(os.path.join(root, name))
    result = [os.path.abspath(p) for p in result]
    if directory:
        if directory not in os.getcwd():
            return [x for x in result if directory in x]
    return result


class MatchStringWith(str):
    """helper class to match sub strings in a mock.assert_called_with

    >>> myMock.log.error.assert_called_with( MatchStringWith('error mess') )
    """

    def __eq__(self, other):
        return self in str(other)



""" MySQL based WMSHistory accounting.
    It's suggested to replace this with the ElasticSearch based WMSHistory monitoring.

    Filled by the agent "WorkloadManagement/StatesAccountingAgent"
"""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

__RCSID__ = "$Id$"

from DIRAC.AccountingSystem.Client.Types.BaseAccountingType import BaseAccountingType


class WMSHistory(BaseAccountingType):
    def __init__(self):
        super(WMSHistory, self).__init__()
        self.definitionKeyFields = [
            ("Status", "VARCHAR(128)"),
            ("Site", "VARCHAR(128)"),
            ("User", "VARCHAR(128)"),
            ("UserGroup", "VARCHAR(128)"),
            ("JobGroup", "VARCHAR(32)"),
            ("MinorStatus", "VARCHAR(128)"),
            ("ApplicationStatus", "VARCHAR(256)"),
            ("JobSplitType", "VARCHAR(32)"),
        ]
        self.definitionAccountingFields = [
            ("Jobs", "INT UNSIGNED"),
            ("Reschedules", "INT UNSIGNED"),
        ]
        self.bucketsLength = [
            (86400 * 2, 900),  # <2d = 15m
            (86400 * 10, 9000),  # <10d = 2.5h
            (86400 * 35, 18000),  # <35d = 5h
            (86400 * 30 * 6, 86400),  # >5d <6m = 1d
            (86400 * 600, 604800),  # >6m = 1w
        ]
        self.dataTimespan = 86400 * 30 * 14  # Only keep the last 14 months of data
        self.checkType()
        self.setValueByKey("ApplicationStatus", "unset")



""" This is the Job Repository which stores and manipulates DIRAC job metadata in CFG format """
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

__RCSID__ = "$Id$"

from diraccfg import CFG

from DIRAC import gLogger, S_OK, S_ERROR

import os
import time
import tempfile
import shutil


class JobRepository(object):
    def __init__(self, repository=None):
        self.location = repository
        if not self.location:
            if "HOME" in os.environ:
                self.location = "%s/.dirac.repo.rep" % os.environ["HOME"]
            else:
                self.location = "%s/.dirac.repo.rep" % os.getcwd()
        self.repo = CFG()
        if os.path.exists(self.location):
            self.repo.loadFromFile(self.location)
            if not self.repo.existsKey("Jobs"):
                self.repo.createNewSection("Jobs")
        else:
            self.repo.createNewSection("Jobs")
        self.OK = True
        written = self._writeRepository(self.location)
        if not written:
            self.OK = False

    def isOK(self):
        return self.OK

    def readRepository(self):
        return S_OK(self.repo.getAsDict("Jobs"))

    def writeRepository(self, alternativePath=None):
        destination = self.location
        if alternativePath:
            destination = alternativePath
        written = self._writeRepository(destination)
        if not written:
            return S_ERROR("Failed to write repository")
        return S_OK(destination)

    def resetRepository(self, jobIDs=[]):
        if not jobIDs:
            jobs = self.readRepository()["Value"]
            jobIDs = list(jobs)
        paramDict = {"State": "Submitted", "Retrieved": 0, "OutputData": 0}
        for jobID in jobIDs:
            self._writeJob(jobID, paramDict, True)
        self._writeRepository(self.location)
        return S_OK()

    def _writeRepository(self, path):
        handle, tmpName = tempfile.mkstemp()
        written = self.repo.writeToFile(tmpName)
        os.close(handle)
        if not written:
            if os.path.exists(tmpName):
                os.remove(tmpName)
            return written
        if os.path.exists(path):
            gLogger.debug("Replacing %s" % path)
        try:
            shutil.move(tmpName, path)
            return True
        except Exception as x:
            gLogger.error("Failed to overwrite repository.", x)
            gLogger.info("If your repository is corrupted a backup can be found %s" % tmpName)
            return False

    def appendToRepository(self, repoLocation):
        if not os.path.exists(repoLocation):
            gLogger.error("Secondary repository does not exist", repoLocation)
            return S_ERROR("Secondary repository does not exist")
        self.repo = CFG().loadFromFile(repoLocation).mergeWith(self.repo)
        self._writeRepository(self.location)
        return S_OK()

    def addJob(self, jobID, state="Submitted", retrieved=0, outputData=0, update=False):
        paramDict = {"State": state, "Time": self._getTime(), "Retrieved": int(retrieved), "OutputData": outputData}
        self._writeJob(jobID, paramDict, update)
        self._writeRepository(self.location)
        return S_OK(jobID)

    def updateJob(self, jobID, paramDict):
        if self._existsJob(jobID):
            paramDict["Time"] = self._getTime()
            self._writeJob(jobID, paramDict, True)
            self._writeRepository(self.location)
        return S_OK()

    def updateJobs(self, jobDict):
        for jobID, paramDict in jobDict.items():
            if self._existsJob(jobID):
                paramDict["Time"] = self._getTime()
                self._writeJob(jobID, paramDict, True)
        self._writeRepository(self.location)
        return S_OK()

    def _getTime(self):
        runtime = time.ctime()
        return runtime.replace(" ", "_")

    def _writeJob(self, jobID, paramDict, update):
        jobID = str(jobID)
        jobExists = self._existsJob(jobID)
        if jobExists and (not update):
            gLogger.warn("Job exists and not overwriting")
            return S_ERROR("Job exists and not overwriting")
        if not jobExists:
            self.repo.createNewSection("Jobs/%s" % jobID)
        for key, value in paramDict.items():
            self.repo.setOption("Jobs/%s/%s" % (jobID, key), value)
        return S_OK()

    def removeJob(self, jobID):
        res = self.repo["Jobs"].deleteKey(str(jobID))  # pylint: disable=no-member
        if res:
            self._writeRepository(self.location)
        return S_OK()

    def existsJob(self, jobID):
        return S_OK(self._existsJob(jobID))

    def _existsJob(self, jobID):
        return self.repo.isSection("Jobs/%s" % jobID)

    def getLocation(self):
        return S_OK(self.location)

    def getSize(self):
        return S_OK(len(self.repo.getAsDict("Jobs")))
